#include "StdAfx.h"

#include "tableparser.h"
#include "orz/file.h"
#include "convert.h"

namespace App
{
	void TableParser::WriteString(std::wstring filename, std::wstring str, OutputFormat format)
	{
		std::FILE* file = 0;
		_wfopen_s(&file, filename.c_str(), TX("wb"));
		COND_STRICT(file, Err::CriticalError, TX("Couldn't open file for writing!"));

		// Convert and write any BOM's
		switch(format)
		{
		case UTF16LE:
			{
				const unsigned char BOM[2] = {0xff, 0xfe};
				fwrite(BOM, 2, 1, file);
				fwrite(str.c_str(), str.length(), sizeof(wchar_t), file);
				break;
			}

		case ANSI:
			{
				std::string ansi = Uni::ConvertToANSI(str);
				fwrite(ansi.c_str(), ansi.length(), sizeof(char), file);
				break;
			}
		}

		fclose(file);
	}

	bool TableParser::Write()
	{
		// Construct the strings
		std::wstring strOut;
		strOut += TX("enum ") + m_enum.Name + TX(" {\n");

		for(size_t i = 0; i < m_enums.size(); ++i)
		{
			strOut += TX("\t") + m_enums[i] + TX("=") + ToString(i) + TX(",\n");
		}

		strOut += TX("};\n");

		WriteString(m_enum.Filename, strOut, m_enum.Format);

		strOut = m_type + TX(" ") + m_table.Name + TX("[") + ToString(m_elements.size()) + TX("]={\n");

		for(size_t i = 0; i < m_elements.size(); ++i)
		{
			strOut += TX("\t") + m_elements[i] + TX(",\n");
		}
		strOut += TX("};\n");

		WriteString(m_table.Filename, strOut, m_table.Format);
		return true;
	}

	TableParser::TableParser(const std::wstring input)
	{
		IO::FileReader file(input);
		file.Open();

		size_t bufferSize = static_cast<size_t>(file.Size());

		if (bufferSize == 0) return;

		boost::scoped_array<char> tmp(new char[bufferSize + 1]);

		file.ReadFull(tmp.get(), bufferSize);
		tmp[bufferSize] = 0;

		std::wstringstream stream(Uni::StripCarriageReturns(Uni::ConvertToWide(tmp.get())));

		m_enum.Load(stream);
		m_table.Load(stream);

		std::getline(stream, m_type);

		// Start parsing da thingamagogs that should go into the table
		std::wstring currentToken;
		do
		{
			do std::getline(stream, currentToken); while (currentToken.size() == 0);
			currentToken = currentToken.substr(currentToken.find_first_not_of(TX(" \t")), std::wstring::npos);

			if(currentToken[0] == TX('!'))
			{
				// This is a terminating enum. Quit after this one
				size_t enumEnd = currentToken.find_first_of(TX(" \t\n"));
				m_enums.push_back(currentToken.substr(1, enumEnd - 2));
				break;
			}
			else if(currentToken[0] != TX('*'))
			{
				// Split up into element ID and data
				size_t enumEnd = currentToken.find_first_of(TX(" \t"));
				size_t elemStart = (enumEnd != std::wstring::npos ?
					currentToken.find_first_not_of(TX(" \t"), enumEnd + 1):
					std::wstring::npos);

				std::wstring enumName = currentToken.substr(0, enumEnd);
				std::wstring data = currentToken.substr(elemStart, std::wstring::npos);
				m_enums.push_back(enumName);
				m_elements.push_back(data);
			}
		} while (currentToken[0] != TX('*'));
	}

	TableParser::~TableParser() {
	}
};
