#include "StdAfx.h"


#include "COM_PropertyHandlerFactory.h"

// {23CC258F-9CEE-4d2f-97CB-8B3542ABCC7A}
static const GUID CLSID_PictPropertyStore = 
{ 0x23cc258f, 0x9cee, 0x4d2f, { 0x97, 0xcb, 0x8b, 0x35, 0x42, 0xab, 0xcc, 0x7a } };

#define SZ_PROPERTYHANDLERSPATH           L"Software\\Microsoft\\Windows\\CurrentVersion\\PropertySystem\\PropertyHandlers"
#define SZ_APPROVEDSHELLEXTENSIONS        L"Software\\Microsoft\\Windows\\CurrentVersion\\Shell Extensions\\Approved"
#define SZ_RECIPEPROPERTYHANDLER          L"Pictus Property Handler"
#define SZ_CLSID_RECIPEPROPERTYHANDLER    L"{23CC258F-9CEE-4d2f-97CB-8B3542ABCC7A}"
#define SZ_FILE_EXTENSION				  L".png"

PropertyHandlerFactory g_propertyHandlerFactory;

LONG	g_cLocks = 0;
HMODULE	g_hmodThis;

//BOOL APIENTRY DllMain(HINSTANCE hModule, DWORD ul_reason_for_call, LPVOID lpReserved)
BOOL APIENTRY DllMain(HINSTANCE hModule, DWORD , LPVOID )
{
	g_hmodThis = (HMODULE)hModule;
	return TRUE;
}

STDAPI DllCanUnloadNow()
{
	return (g_cLocks == 0) ? S_OK : S_FALSE;
}

void DllAddRef()
{
	InterlockedIncrement(&g_cLocks);
}

void DllRelease()
{
	InterlockedDecrement(&g_cLocks);
}

STDAPI DllGetClassObject(REFCLSID clsid, REFIID riid, void **ppv)
{
	HRESULT hr = CLASS_E_CLASSNOTAVAILABLE;

	if (clsid == CLSID_PictPropertyStore)
	{
		hr = g_propertyHandlerFactory.QueryInterface(riid, ppv);
	}

	return hr;
}

struct REGISTRY_ENTRY
{
	HKEY    hkeyRoot;
	LPCWSTR pszKeyName;
	LPCWSTR pszValueName;
	DWORD   dwType;
	LPCWSTR pszData;
	DWORD   dwData;
};


HRESULT CreateRegKeyAndSetValue(REGISTRY_ENTRY *pRegistryEntry)
{
	HRESULT hr = E_INVALIDARG;

	if (pRegistryEntry != NULL)
	{
		// create the key, or obtain its handle if it already exists
		HKEY hKey;
		LONG lr = RegCreateKeyExW(pRegistryEntry->hkeyRoot,
			pRegistryEntry->pszKeyName, 
			0, 
			NULL, 
			REG_OPTION_NON_VOLATILE,
			KEY_ALL_ACCESS, 
			NULL, 
			&hKey, 
			NULL);
		hr = HRESULT_FROM_WIN32(lr);
		if (SUCCEEDED(hr))
		{
			// extract the data from the struct according to its type
			LPBYTE pData = NULL;
			DWORD cbData = 0;
			hr = S_OK;
			switch (pRegistryEntry->dwType)
			{
			case REG_SZ:
				pData = (LPBYTE) pRegistryEntry->pszData;
				cbData = ((DWORD) wcslen(pRegistryEntry->pszData) + 1) * sizeof(WCHAR);
				break;

			case REG_DWORD:
				pData = (LPBYTE) &pRegistryEntry->dwData;
				cbData = sizeof(pRegistryEntry->dwData);
				break;

			default:
				hr = E_INVALIDARG;
				break;
			}

			if (SUCCEEDED(hr))
			{
				// attempt to set the value
				lr = RegSetValueExW(hKey,
					pRegistryEntry->pszValueName,
					0,
					pRegistryEntry->dwType,
					pData,
					cbData);
				hr = HRESULT_FROM_WIN32(lr);
			}

			RegCloseKey(hKey);
		}
	}

	return hr;
}

STDAPI DllRegisterServer()
{
	HRESULT hr = E_FAIL;

	WCHAR szModuleName[MAX_PATH];

	if (!GetModuleFileNameW(g_hmodThis, szModuleName, ARRAYSIZE(szModuleName)))
	{
		hr = HRESULT_FROM_WIN32(GetLastError());
	}
	else
	{
		//
		// List of property-handler specific registry entries we need to create
		//
		REGISTRY_ENTRY rgRegistryEntries[] =
		{
			//
			// COM information
			//
			{
				HKEY_CLASSES_ROOT,
					L"CLSID\\" SZ_CLSID_RECIPEPROPERTYHANDLER,
					NULL,
					REG_SZ,
					SZ_RECIPEPROPERTYHANDLER,
					0
			},
			{
				HKEY_CLASSES_ROOT,
					L"CLSID\\" SZ_CLSID_RECIPEPROPERTYHANDLER,
					L"ManualSafeSave",
					REG_DWORD,
					NULL,
					1
				},
				{
					HKEY_CLASSES_ROOT,
						L"CLSID\\" SZ_CLSID_RECIPEPROPERTYHANDLER L"\\InProcServer32",
						NULL,
						REG_SZ,
						szModuleName,
						0
				},
				{
					HKEY_CLASSES_ROOT,
						L"CLSID\\" SZ_CLSID_RECIPEPROPERTYHANDLER L"\\InProcServer32",
						L"ThreadingModel",
						REG_SZ,
						L"Apartment",
						0
					},

					//
					// Shell information
					//
					{
						HKEY_LOCAL_MACHINE,
							SZ_PROPERTYHANDLERSPATH L"\\" SZ_FILE_EXTENSION,
							NULL,
							REG_SZ,
							SZ_CLSID_RECIPEPROPERTYHANDLER,
							0
					},
					{
						HKEY_LOCAL_MACHINE,
							SZ_APPROVEDSHELLEXTENSIONS,
							SZ_CLSID_RECIPEPROPERTYHANDLER,
							REG_SZ,
							SZ_RECIPEPROPERTYHANDLER,
							0
						}
		};

		hr = S_OK;
		for (int i = 0; i < ARRAYSIZE(rgRegistryEntries) && SUCCEEDED(hr); i++)
		{
			hr = CreateRegKeyAndSetValue(&rgRegistryEntries[i]);
		}

		if (SUCCEEDED(hr))
		{
			// inform Explorer, et al of the new handler
			SHChangeNotify(SHCNE_ASSOCCHANGED, SHCNF_IDLIST, 0, 0);
		}
	}

	return hr;
}


//
// Unregisters this COM server
//
STDAPI DllUnregisterServer()
{
	HRESULT hr = S_OK;

	// attempt to delete everything, even if some operations fail
	DWORD dwCLSID =           SHDeleteKeyW(HKEY_CLASSES_ROOT, L"CLSID\\" SZ_CLSID_RECIPEPROPERTYHANDLER);
	DWORD dwPropertyHandler = SHDeleteKeyW(HKEY_LOCAL_MACHINE, SZ_PROPERTYHANDLERSPATH L"\\" SZ_FILE_EXTENSION);
	DWORD dwShellExtension =  SHDeleteValueW(HKEY_LOCAL_MACHINE, SZ_APPROVEDSHELLEXTENSIONS, SZ_CLSID_RECIPEPROPERTYHANDLER);

	// return first error encountered as HRESULT
	if (dwCLSID != ERROR_SUCCESS)
	{
		hr = HRESULT_FROM_WIN32(dwCLSID);
	}
	else if (dwPropertyHandler != ERROR_SUCCESS)
	{
		hr = HRESULT_FROM_WIN32(dwPropertyHandler);
	}
	else if (dwShellExtension != ERROR_SUCCESS)
	{
		hr = HRESULT_FROM_WIN32(dwShellExtension);
	}

	if (SUCCEEDED(hr))
	{
		// inform Explorer, et al that the handler has been unregistered
		SHChangeNotify(SHCNE_ASSOCCHANGED, SHCNF_IDLIST, 0, 0);
	}

	return hr;
}



//
// Gets the path to the schema file, assuming it is in the same directory as this DLL.
//
HRESULT GetSchemaPath(LPWSTR pwszSchemaPath, UINT cchSchemaPath)
{
	HRESULT hr = E_INVALIDARG;

	if (pwszSchemaPath && cchSchemaPath >= MAX_PATH)
	{
		// get the absolute path to this dll, remove the dll filename, and append the schema filename
		hr = E_FAIL;
		if (GetModuleFileNameW(g_hmodThis, pwszSchemaPath, cchSchemaPath) &&
			PathRemoveFileSpecW(pwszSchemaPath) &&
			PathAppendW(pwszSchemaPath, L"RecipeProperties.propdesc"))
		{
			hr = S_OK;
		}
	}

	return hr;
}


//
// Installs custom .recipe properties on the system; assumes RecipeProperties.propdesc is in the same directory as this DLL.
//
STDAPI InstallPropertySchema()
{
	HRESULT hr = CoInitialize(NULL);
	if (SUCCEEDED(hr))
	{
		WCHAR wszSchemaPath[MAX_PATH] = {0};
		hr = GetSchemaPath(wszSchemaPath, ARRAYSIZE(wszSchemaPath));
		if (SUCCEEDED(hr))
		{
			hr = PSRegisterPropertySchema(wszSchemaPath);
		}

		CoUninitialize();
	}

	return hr;
}


//
// Uninstalls custom .recipe properties from the system; assumes RecipeProperties.propdesc is in the same directory as this DLL.
//
STDAPI UninstallPropertySchema()
{
	HRESULT hr = CoInitialize(NULL);
	if (SUCCEEDED(hr))
	{
		WCHAR wszSchemaPath[MAX_PATH] = {0};
		hr = GetSchemaPath(wszSchemaPath, ARRAYSIZE(wszSchemaPath));
		if (SUCCEEDED(hr))
		{
			hr = PSUnregisterPropertySchema(wszSchemaPath);
		}

		CoUninitialize();
	}

	return hr;
}
