#include "StdAfx.h"
#include "regsetup.h"
#include "codecsetup.h"
#include "dllmain.h"

struct REGISTRY_ENTRY {
	HKEY    hkeyRoot;
	WString pszKeyName;
	WChar* pszValueName;
	WChar* pszData;
};

// Creates a registry key (if needed) and sets the default value of the key

HRESULT CreateRegKeyAndSetValue(const REGISTRY_ENTRY *pRegistryEntry) {
	HRESULT hr;

	HKEY hKey;

	LONG lRet = RegCreateKeyExW(pRegistryEntry->hkeyRoot, pRegistryEntry->pszKeyName.c_str(), 
		0, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hKey, NULL);

	if (lRet != ERROR_SUCCESS)
		hr = HRESULT_FROM_WIN32(lRet);
	else {
		lRet = RegSetValueExW(hKey, pRegistryEntry->pszValueName, 0, REG_SZ,
			(LPBYTE) pRegistryEntry->pszData,
			((DWORD) wcslen(pRegistryEntry->pszData) + 1) * sizeof(WCHAR));

		hr = HRESULT_FROM_WIN32(lRet);

		RegCloseKey(hKey);
	}

	return hr;
}


HRESULT RegisterThumbnailProvider(const String& extension) {
	// List of registry entries we want to create
	const REGISTRY_ENTRY rgRegistryEntries[] = {
		// RootKey				KeyName														ValueName						Data
		{HKEY_CLASSES_ROOT,		L"." + extension,											L"TypeOverlay",					L""},
		{HKEY_CLASSES_ROOT,		L"." + extension,											L"PerceivedType",				L"image"},
		{HKEY_CLASSES_ROOT,		L"." + extension + L"\\ShellEx\\{e357fccd-a995-4576-b01f-234630154e96}",	NULL,			SZ_CLSID_PICTTHUMBSPROVIDER},
	};

	HRESULT hr = S_OK;

	for (int i = 0; i < ARRAYSIZE(rgRegistryEntries) && SUCCEEDED(hr); i++) {
		hr = CreateRegKeyAndSetValue(&rgRegistryEntries[i]);
	}
	return hr;
}

HRESULT RegistrySetup() {
	WCHAR szModuleName[MAX_PATH];

	if (!GetModuleFileNameW(g_hInst, szModuleName, ARRAYSIZE(szModuleName)))
		return HRESULT_FROM_WIN32(GetLastError());

	const REGISTRY_ENTRY rgRegistryEntries[] = {
		{HKEY_CLASSES_ROOT,		L"CLSID\\" SZ_CLSID_PICTTHUMBSPROVIDER,						NULL,							SZ_THUMBNAILPROVIDERNAME},
		{HKEY_CLASSES_ROOT,		L"CLSID\\" SZ_CLSID_PICTTHUMBSPROVIDER L"\\InProcServer32",	NULL,							szModuleName},
		{HKEY_CLASSES_ROOT,		L"CLSID\\" SZ_CLSID_PICTTHUMBSPROVIDER L"\\InProcServer32",	L"ThreadingModel",				L"Apartment"},
	};
	HRESULT hr = S_OK;

	for (int i = 0; i < ARRAYSIZE(rgRegistryEntries) && SUCCEEDED(hr); i++) {
		hr = CreateRegKeyAndSetValue(&rgRegistryEntries[i]);
	}

	if(FAILED(hr))
		return hr;

	Img::CodecFactoryStore cfs;
	CodecManagerSetup(&cfs);

	const Img::CodecFactoryStore::InfoVector& v = cfs.CodecInfo();
	for(size_t i = 0; i < v.size(); i++) {
		const Img::CodecFactoryStore::Info& info = v[i];
		for(size_t j = 0; j < info.Extensions.size(); ++j) {
			HRESULT hr = RegisterThumbnailProvider(info.Extensions[j]);
			if(FAILED(hr)) return hr;
		}
	}

	SHChangeNotify(SHCNE_ASSOCCHANGED, SHCNF_IDLIST, NULL, NULL);
	return hr;
}

HRESULT UnRegisterThumbnailProvider(const String& extension) {
	WString extKey = L"." + extension + L"\\ShellEx\\{e357fccd-a995-4576-b01f-234630154e96}";

	return HRESULT_FROM_WIN32(SHDeleteKeyW(HKEY_CLASSES_ROOT, extKey.c_str()));
}

HRESULT RegistryCleanup() {
	Img::CodecFactoryStore cfs;
	CodecManagerSetup(&cfs);

	const PCWSTR clsid = L"CLSID\\" SZ_CLSID_PICTTHUMBSPROVIDER;

	DWORD dwError = SHDeleteKeyW(HKEY_CLASSES_ROOT, clsid);
	HRESULT hr = HRESULT_FROM_WIN32(dwError);
	if(hr == HRESULT_FROM_WIN32(ERROR_FILE_NOT_FOUND))
		hr = S_OK;

	if(!SUCCEEDED(hr))
		return hr;

	const Img::CodecFactoryStore::InfoVector& v = cfs.CodecInfo();
	for(size_t i = 0; i < v.size(); i++) {
		const Img::CodecFactoryStore::Info& info = v[i];
		for(size_t j = 0; j < info.Extensions.size(); ++j) {
			HRESULT hr = UnRegisterThumbnailProvider(info.Extensions[j]);
			if(hr == HRESULT_FROM_WIN32(ERROR_FILE_NOT_FOUND))
				hr = S_OK;
			if(FAILED(hr)) return hr;
		}
	}
	return S_OK;
}
