#include "StdAfx.h"
#include "assoc_util.h"
#include "illa/codecmgr.h"
#include "res_viewer.h"
#include "assoc_vista.h"
#include "assoc_xp_power.h"
#include "association_interface.h"
#include "orz/sysinfo.h"
#include "Config.h"

namespace App {
	namespace Association {
		//Mode g_mode = AMUndefined;
		AssociationInterface g_if;
		Img::CodecFactoryStore* g_cfs;

		AssociationMode::Ptr doConstruct(Mode m) {
			typedef AssociationMode::Ptr Ptr;

			switch(m) {
			case AMVista:
				return Ptr(new AssociationModeVista());
			case AMXP:
				return Ptr(new AssociationModeXpPowerUser());
				//			case AMLegacyStandardUser:
				//				return Ptr(new AssociationModeXpStandardUser());

			case AMUndefined:
				DO_THROW(Err::InvalidParam, TX("AssociationMode not valid (AMUndefined"));
			default:
				DO_THROW(Err::NotYetImplemented, TX(""));
			}
		}

		AssociationMode::Ptr ConstructMode(Mode m, bool isInstaller) {
			AssociationMode::Ptr p(doConstruct(m));
			p->IsLaunchedThroughInstaller(isInstaller);
			return p;
		}

		FileFormat::Ptr ConstructFileFormat(size_t index) {
			const Img::CodecFactoryStore::InfoVector& info(g_cfs->CodecInfo());
			COND_STRICT(index < info.size(), Err::InvalidParam, TX("index was out of range (too large)."));

			FileFormat::Ptr ft(new FileFormat(
				TX("Pictus.") + info[index].Extensions[0],
				info[index].Description,
				IDI_DOCUMENT));
			//ft->SetIcon(IDI_DOCUMENT);

			const Img::CodecFactoryStore::Info::ExtensionVector& v = info[index].Extensions;
			for (size_t j = 0; j < v.size(); ++j)
				ft->AddExtension(v[j]);

			return ft;
		}

		void SetAssociationsMode(Mode m, bool isInstaller) {
			g_if.Reset(cg_PictusDefaultProgramsName, Intl::GetString(SIDPictusDescription));
			g_if.SetMode(ConstructMode(m, isInstaller));
			size_t count = g_cfs->CodecInfo().size();
			for (size_t i = 0; i < count; ++i)
				g_if.AddFileFormat(ConstructFileFormat(i));
		}

		bool HasSetupMachineLevelRegistry() {
			return g_if.HasSetupMachineLevelRegistry();
		}

		void SetupMachineLevelRegistry() {
			g_if.SetupMachineLevelRegistry(Sys::Info::ExePath());
		}


		void AssociateAll() {
			for (size_t i = 0; i < g_cfs->CodecInfo().size(); ++i)
				Associate(i, true);
			Commit();
		}

		void AssociateNone() {
			for(size_t i = 0; i < g_cfs->CodecInfo().size(); ++i)
				Associate(i, false);
			Commit();
		}

		void Associate(size_t index, bool doSet) {
			if(doSet)
				g_if.Associate(ConstructFileFormat(index));
			else
				g_if.UnAssociate(ConstructFileFormat(index));
		}

		void Commit() {
			g_if.Commit();
		}

		bool IsAssociated(size_t index) {
			const Img::CodecFactoryStore::InfoVector& info(g_cfs->CodecInfo());
			COND_STRICT(index < info.size(), Err::InvalidParam, TX("index was out of range (too large)."));

			return g_if.IsFileFormatAssociated(ConstructFileFormat(index));
		}

		void DeleteMachineLevelRegistry() {
			g_if.DeleteMachineLevelRegistry();
		}

		void DeleteDeprecatedAssociations() {
			g_if.DeleteDeprecatedAssociations();
		}

		void SetCodecFactoryStore(Img::CodecFactoryStore* cfs) {
			g_cfs = cfs;
		}
	};
}
