#include "StdAfx.h"
#include "assoc_vista.h"
#include "w32_reg.h"
#include "reg_defaultprograms.h"
#include "reg_progid.h"

namespace App {
	const String AssociationModeVista::ClassesPath(TX("Software\\Classes\\"));

	bool AssociationModeVista::HasWrittenGlobalData() {
		return DefaultPrograms::IsRegistered();
	}

	bool AssociationModeVista::HasWrittenProgId(FileFormat::Ptr fmt) {
		return ProgId::HasWrittenProgId(fmt, false);
	}

	void AssociationModeVista::OnBeginWriteProgIds(const String& name, const String& description) {
		if(IsLaunchedThroughInstaller())
			DefaultPrograms::Register(name, description);
	}

	void AssociationModeVista::OnWriteProgId(FileFormat::Ptr fmt, const String& executable) {
		bool doHKCU = !IsLaunchedThroughInstaller();

		ProgId::WriteProgId(fmt, executable, doHKCU);
		if(doHKCU == false)
			ProgId::DeleteProgId(fmt, true);

		if(!doHKCU) {
			foreach(String ext, fmt->Extensions())
				DefaultPrograms::AddExtension(fmt->Identifier(), ext);
		}
		else {
			foreach(const String& s, fmt->Extensions()) {
				CreateExtension(s);
			}
		}
	}

	void AssociationModeVista::OnEndWriteProgIds() {}


	bool AssociationModeVista::OnIsAssociated(FileFormat::Ptr fmt) {
		foreach(String ext, fmt->Extensions())
			if(!DefaultPrograms::IsAssociated(ext)) return false;
		return true;
	}

	void AssociationModeVista::OnAssociate(FileFormat::Ptr fmt) {
		foreach(String ext, fmt->Extensions())
			DefaultPrograms::Associate(ext);
	}

	void AssociationModeVista::OnUnAssociate(FileFormat::Ptr fmt) {
		//DefaultPrograms::UnAssociate(fmt->Extensions()[0]);
	}

	void AssociationModeVista::DeleteGlobalData() {
		DefaultPrograms::Unregister();
	}

	void AssociationModeVista::DeleteProgId(FileFormat::Ptr fmt) {
		bool doHKCU = !IsLaunchedThroughInstaller();
		if(!doHKCU)
			ProgId::DeleteProgId(fmt, false);
		ProgId::DeleteProgId(fmt, true);
	}

	void AssociationModeVista::Commit() {
	}

	void AssociationModeVista::CreateExtension(const String& ext) {
		Win::RegistryKey key_base(Win::RegistryKey::CurrentUser, (ClassesPath + TX(".") + ToLower(ext)));
		key_base.Open(true);
	}

	void AssociationModeVista::OnDeleteDeprecatedAssociation(FileFormat::Ptr fmt) {
		ProgId::DeleteDeprecatedAssociation(fmt);
	}
}
