#include "StdAfx.h"
#include "assoc_xp_power.h"
#include "w32_reg.h"
#include "reg_progid.h"
#include "reg_defaultprograms.h"

namespace App {
	const String AssociationModeXpPowerUser::ClassesPath(TX("Software\\Classes\\"));

	bool AssociationModeXpPowerUser::HasWrittenGlobalData() {
		return DefaultPrograms::IsRegistered();
	}

	bool AssociationModeXpPowerUser::HasWrittenProgId(FileFormat::Ptr fmt) {
		return ProgId::HasWrittenProgId(fmt, false);
	}

	bool AssociationModeXpPowerUser::OnIsAssociated(FileFormat::Ptr fmt) {
		return ProgId::IsAssociated(fmt);
	}

	void AssociationModeXpPowerUser::OnBeginWriteProgIds(const String& name, const String& description) {
		if(IsLaunchedThroughInstaller())
			DefaultPrograms::Register(name, description);
	}

	void AssociationModeXpPowerUser::OnWriteProgId(FileFormat::Ptr fmt, const String& executable) {
		bool doHKCU = !IsLaunchedThroughInstaller();

		ProgId::WriteProgId(fmt, executable, doHKCU);
		if(doHKCU == false)
			ProgId::DeleteProgId(fmt, true);

		foreach(const String& s, fmt->Extensions()) {
			CreateExtension(s);
			if(doHKCU == false) DefaultPrograms::AddExtension(fmt->Identifier(), s);
		}
	}

	void AssociationModeXpPowerUser::OnEndWriteProgIds() {}

	void AssociationModeXpPowerUser::CreateExtension(const String& ext) {
		Win::RegistryKey key_base(Win::RegistryKey::CurrentUser, (ClassesPath + TX(".") + ToLower(ext)));
		key_base.Open(true);
	}

	void AssociationModeXpPowerUser::OnAssociate(FileFormat::Ptr fmt) {
		ProgId::Associate(fmt);
	}

	void AssociationModeXpPowerUser::OnUnAssociate(FileFormat::Ptr fmt) {
		ProgId::UnAssociate(fmt);
	}

	void AssociationModeXpPowerUser::DeleteGlobalData() {
		if(IsLaunchedThroughInstaller()) DefaultPrograms::Unregister();
	}

	void AssociationModeXpPowerUser::DeleteProgId(FileFormat::Ptr fmt) {
		ProgId::DeleteProgId(fmt, !IsLaunchedThroughInstaller());
	}

	void AssociationModeXpPowerUser::Commit() {
	}

	void AssociationModeXpPowerUser::OnDeleteDeprecatedAssociation(FileFormat::Ptr fmt) {
		ProgId::DeleteDeprecatedAssociation(fmt);
	}
}
