#include "StdAfx.h"
#include "association_interface.h"

namespace App {
	void AssociationInterface::Reset(const String& applicationName, const String& applicationDescription) {
		m_mode.reset();
		m_formats.clear();
		m_appName = applicationName;
		m_appDesc = applicationDescription;
	}

	void AssociationInterface::SetMode(AssociationMode::Ptr mode) {
		m_mode = mode;
	}

	void AssociationInterface::AddFileFormat(FileFormat::Ptr fmt) {
		m_formats.push_back(fmt);
	}

	bool AssociationInterface::HasSetupMachineLevelRegistry() {
		if (!m_mode->HasWrittenGlobalData()) return false;
		foreach(FileFormat::Ptr p, m_formats)
			if(m_mode->HasWrittenProgId(p) == false) return false;

		return true;
	}

	void AssociationInterface::SetupMachineLevelRegistry(const String& executable) {
		m_mode->BeginWriteProgIds(m_appName, m_appDesc);
		foreach(FileFormat::Ptr p, m_formats)
			m_mode->WriteProgId(p, executable);
		m_mode->EndWriteProgIds();
	}

	bool AssociationInterface::IsFileFormatAssociated(FileFormat::Ptr fmt) {
		return m_mode->IsAssociated(fmt);
	}

	void AssociationInterface::Associate(FileFormat::Ptr fmt) {
		return m_mode->Associate(fmt);
	}

	void AssociationInterface::UnAssociate(FileFormat::Ptr fmt) {
		return m_mode->UnAssociate(fmt);
	}

	void AssociationInterface::DeleteMachineLevelRegistry() {
		if(m_mode->HasWrittenGlobalData())
			m_mode->DeleteGlobalData();
		foreach(FileFormat::Ptr p, m_formats)
			m_mode->DeleteProgId(p);
	}

	void AssociationInterface::DeleteDeprecatedAssociations() {
		foreach(FileFormat::Ptr p, m_formats)
			m_mode->DeleteDeprecatedAssociation(p);
	}               

	void AssociationInterface::Commit() {
		m_mode->Commit();
	}
}
