#include "StdAfx.h"
#include "common_controls.h"
#include "assoc_util.h"

namespace App {
	using namespace Intl;

	void FileFormatsListView::UpdateStrings() {
		m_filetypes->ColumnName(FileNameColumn, GetString(SIDTypeName));
		m_filetypes->ColumnName(FileExtColumn, GetString(SIDTypeExtension));
	}

	FileFormatsListView::FileFormatsListView(Win::Dialog* pDialog, DWORD id, Img::CodecFactoryStore* cfs)
		:m_filetypes(pDialog->CreateListView(id)),
		m_cfs(cfs)
	{
		m_lang = Intl::OnLanguageChanged.connect(bind(&FileFormatsListView::UpdateStrings, this));

		m_filetypes->Clear();
		m_filetypes->Style(LVS_EX_CHECKBOXES | LVS_EX_FULLROWSELECT);
		m_filetypes->AddColumn(GetString(SIDTypeName), 200, FileNameColumn);
		m_filetypes->AddColumn(GetString(SIDTypeExtension), 130, FileExtColumn);

		const Img::CodecFactoryStore::InfoVector& info = cfs->CodecInfo();

		for (size_t i = 0; i < info.size(); ++i) {
			int index = m_filetypes->AddItem(info[i].Description);

			const Img::CodecFactoryStore::Info::ExtensionVector& v = info[i].Extensions;

			String exts = v[0];
			for (unsigned j = 1; j < info[i].Extensions.size(); ++j)
				exts += TX(", ") + v[j];
			m_filetypes->ItemColumn(index, 1, exts);
		}

		for (size_t i = 0; i < info.size(); ++i)
			m_filetypes->Check(i, App::Association::IsAssociated(i));
	}

	void FileFormatsListView::WriteAssociations() {
		const Img::CodecFactoryStore::InfoVector& info = m_cfs->CodecInfo();
		bool anyChange = false;

		for (size_t i = 0; i < info.size(); ++i) {
			if (m_filetypes->IsChecked(i) != App::Association::IsAssociated(i))
				anyChange = true;

			App::Association::Associate(i, m_filetypes->IsChecked(i));
		}

		// Avoid flashing if there's no changes.
		if (anyChange)
			App::Association::Commit();
	}

}
