#include "StdAfx.h"
#include "dlg_view.h"
#include "registry.h"
#include "res_settings.h"
#include "orz/intl.h"
#include "ctrl_button.h"

namespace App {
	using namespace Reg::Keys;
	using namespace Intl;

	void SetView::ToggleResizeWindow() {
		DoToggleResizeWindow(IsDlgButtonChecked(Handle(), IDC_SIZETOIMAGE)!=0);
	}

	bool SetView::PerformOnInitPage() {
		CreateButton(IDC_SIZETOIMAGE)->OnClick.connect(bind(&SetView::ToggleResizeWindow, this));
		Caption(App::SIDViewer);

		m_cbDefaultZoom = CreateComboBox(IDC_COMBO_VIEWER_DEFAULTZOOM);
		m_cbResizeBehavior = CreateComboBox(IDC_COMBO_VIEWER_RESIZEBEHAVIOUR);
		m_cbMinFilter = CreateComboBox(IDC_COMBO_MIN);
		m_cbMagFilter = CreateComboBox(IDC_COMBO_MAG);
		m_cbRenderer = CreateComboBox(IDC_COMBO_VIEWER_RENDERER);

		ControlText(IDC_GROUP_ZOOMING, SIDZoom);
		ControlText(IDC_GROUP_MAG, SIDGroupMagnification);
		ControlText(IDC_GROUP_MIN, SIDGroupMinification);
		ControlText(IDC_GROUP_VIEWER, SIDGroupViewer);
		ControlText(IDC_RESETZOOM, SIDResetZoom);
		ControlText(IDC_RESETPAN, SIDResetPan);
		ControlText(IDC_SIZETOIMAGE, SIDResizeWindow);
		ControlText(IDC_SIZETO_SCREEN, SIDWindowCenterScreen);
		ControlText(IDC_SIZETO_CURRENT, SIDWindowCenterCurrent);
		ControlText(IDC_SIZETO_NOTHING, SIDWindowNoReposition);
		ControlText(IDC_CHECK_VIEWER_WRAPAROUND, SIDWrapAround);

		ControlText(IDC_GROUP_VIEWER_DEFAULTZOOM, SIDGroupZM);

		ControlText(IDC_GROUP_VIEWER_RENDERING, SIDRenderer);
		ControlText(IDC_TEXT_VIEWER_RESTART, SIDRendererRestart);

		m_cbDefaultZoom->Reset();	//SendDlgItemMessage(Handle(), IDC_COMBO_VIEWER_DEFAULTZOOM, CB_RESETCONTENT, 0, 0); 
		m_cbDefaultZoom->AddItem(SIDZoomFullSize, ZoomFree);
		m_cbDefaultZoom->AddItem(SIDZoomFitImage, ZoomFitImage);
		m_cbDefaultZoom->SetSelection(Reg::Key(DWDefaultZoomMode));

		ControlText(IDC_GROUP_VIEWER_RESIZEBEHAVIOUR, SIDGroupRM);
		m_cbResizeBehavior->Reset(); //SendDlgItemMessage(Handle(), IDC_COMBO_VIEWER_RESIZEBEHAVIOUR, CB_RESETCONTENT, 0, 0); 
		m_cbResizeBehavior->AddItem(SIDRMBoth, ResizeEnlargeOrReduce);
		m_cbResizeBehavior->AddItem(SIDRMEnlargeOnly, ResizeEnlargeOnly);
		m_cbResizeBehavior->AddItem(SIDRMReduceOnly, ResizeReduceOnly);
		m_cbResizeBehavior->SetSelection(Reg::Key(DWResizeBehaviour));

		SetupFilterBox(m_cbMagFilter);
		m_cbMagFilter->SetSelection(Reg::Key(DWMagFilter));

		SetupFilterBox(m_cbMinFilter);
		m_cbMinFilter->SetSelection(Reg::Key(DWMinFilter));
		SetupRendererBox(m_cbRenderer);
		m_cbRenderer->SetSelection(Reg::Key(DWRenderer));

		SetCheckBox(IDC_CHECK_VIEWER_WRAPAROUND, Reg::Key(DWBrowseWrapAround) != 0);
		SetCheckBox(IDC_RESETZOOM, Reg::Key(DWResetZoom) != 0);
		SetCheckBox(IDC_RESETPAN,  Reg::Key(DWResetPan) != 0);
		DoToggleResizeWindow(Reg::Key(DWResizeWindow) != 0);

		// Position method
		DWORD posmethod=Reg::Key(DWResizePositionMethod);
		if (posmethod == PositionToScreen)
			SetCheckBox(IDC_SIZETO_SCREEN, true);

		if (posmethod == PositionToCurrent)
			SetCheckBox(IDC_SIZETO_CURRENT, true);

		if (posmethod == PositionNothing)
			SetCheckBox(IDC_SIZETO_NOTHING, true);

		m_cbRenderer->Visible(false);
		ShowWindow(GetDlgItem(Handle(), IDC_GROUP_VIEWER_RENDERING), SW_HIDE);
		ShowWindow(GetDlgItem(Handle(), IDC_TEXT_VIEWER_RESTART), SW_HIDE);
		
		return true;
	}

	void SetView::onWriteSettings() {
		Reg::Key(DWBrowseWrapAround, GetCheckBox(IDC_CHECK_VIEWER_WRAPAROUND));
		Reg::Key(DWResetZoom, GetCheckBox(IDC_RESETZOOM));
		Reg::Key(DWResetPan,  GetCheckBox(IDC_RESETPAN));
		Reg::Key(DWResizeWindow, GetCheckBox(IDC_SIZETOIMAGE));
		Reg::Key(DWMinFilter, m_cbMinFilter->GetSelectionData());
		Reg::Key(DWMagFilter, m_cbMagFilter->GetSelectionData());

		Reg::Key(DWDefaultZoomMode, m_cbDefaultZoom->GetSelectionData());
		Reg::Key(DWResizeBehaviour, m_cbResizeBehavior->GetSelectionData());

		DWORD posmethod=Reg::Key(DWResizePositionMethod);
		if (IsDlgButtonChecked(Handle(), IDC_SIZETO_SCREEN))
			posmethod = PositionToScreen;
		else if (IsDlgButtonChecked(Handle(), IDC_SIZETO_CURRENT))
			posmethod = PositionToCurrent;
		else if (IsDlgButtonChecked(Handle(), IDC_SIZETO_NOTHING))
			posmethod = PositionNothing;

		Reg::Key(DWResizePositionMethod, posmethod);
		Reg::Key(DWRenderer, m_cbRenderer->GetSelectionData());
	}

	void SetView::DoToggleResizeWindow(bool newState) {
		SetCheckBox(IDC_SIZETOIMAGE, newState);
		EnableWindow(GetDlgItem(Handle(), IDC_SIZETO_SCREEN), newState);
		EnableWindow(GetDlgItem(Handle(), IDC_SIZETO_CURRENT), newState);
		EnableWindow(GetDlgItem(Handle(), IDC_SIZETO_NOTHING), newState);
	}

	void SetView::SetupFilterBox(Win::ComboBox* ctrl) {
		ctrl->Reset(); //SendDlgItemMessage(Handle(), id, CB_RESETCONTENT, 0, 0); 
		ctrl->AddItem(SIDFilterNearestNeighbor, Filter::FilterNearestNeighbor);
		ctrl->AddItem(SIDFilterBilinear, Filter::FilterBilinear);
		ctrl->AddItem(SIDFilterLanczos3, Filter::FilterLanczos3);
	}

	SetView::SetView():App::SettingsPage(IDD_SET_VIEWER) {}

	void SetView::SetupRendererBox(Win::ComboBox* ctrl) {
		ctrl->Reset(); //SendDlgItemMessage(Handle(), id, CB_RESETCONTENT, 0, 0); 
		ctrl->AddItem(SIDRendererGDI, App::RenderGDI);
		ctrl->AddItem(SIDRendererD3D, App::RenderD3D);
	}	
}
