#include "StdAfx.h"
#include "elevation.h"
#include "osinfo.h"
#include <tchar.h>
#include "w32_reg.h"

namespace Win {
	bool LaunchThisProcessAsElevated(HWND hwnd, const String& cmdParams) {
		TCHAR szPath[ MAX_PATH ];
		::GetModuleFileName( NULL, szPath, MAX_PATH );

		SHELLEXECUTEINFO shex;

		memset( &shex, 0, sizeof( shex) );

		shex.cbSize			= sizeof( SHELLEXECUTEINFO ); 
		shex.fMask			= SEE_MASK_NOCLOSEPROCESS; 
		shex.hwnd			= hwnd;
		shex.lpVerb			= TX("runas"); 
		shex.lpFile			= szPath; 
		shex.lpParameters	= cmdParams.c_str(); 
		shex.lpDirectory	= 0; 
		shex.nShow			= SW_NORMAL; 

		if (::ShellExecuteEx( &shex ) == FALSE) return false;

		if(shex.hProcess)
			WaitForSingleObject(shex.hProcess, INFINITE);
		CloseHandle(shex.hProcess);
		return true;
	}

	bool IsElevated() {
		if (GetWindowsVersion() < WindowsVista) {
			// This is probably a bad way of checking for power user privileges in XP, but I can't figure out
			// how to do it properly.
			Win::RegistryKey k(Win::RegistryKey::LocalMachine, TX("SOFTWARE"));
			return k.Open();
		}

		HANDLE hToken = 0;

		try {
			COND_STRICT(::OpenProcessToken(::GetCurrentProcess(), TOKEN_QUERY, &hToken), Err::CriticalError, TX("OpenProcessToken failed."));

			TOKEN_ELEVATION te = { 0 };
			DWORD dwReturnLength = 0;

			COND_STRICT(::GetTokenInformation(hToken, TokenElevation, &te, sizeof(te), &dwReturnLength), Err::CriticalError, TX("GetTokenInformation failed."));
			CloseHandle(hToken);

			COND_STRICT(dwReturnLength == sizeof(te), Err::CriticalError, TX("Token length mismatch."));

			return te.TokenIsElevated != 0;
		}
		catch(...) {
			::CloseHandle( hToken );
			throw;
		}
	}
}
