
#ifndef IMAGECACHE_WORKERTHREAD_H
#define IMAGECACHE_WORKERTHREAD_H

#include "orz/thread.h"

#include "ic_queue.h"
#include "ic_worklist.h"
#include "imagecache_event.h"

namespace Img {
	class DecoderWorkerThread {
	public:
		void Queue(ImageQueue::Ptr queue);
		void Start();
		void Update(bool wait = false);

		void SetCodecFactoryStore(Img::CodecFactoryStore* cfs);

		IO::FileReader::Ptr GetFileReader(const String& filename);

		void Terminate();

		void MemoryLimit(size_t newLimit);

		void MessageTarget(MessageReceiver* receiver);

		DecoderWorkerThread();
		~DecoderWorkerThread();

		typedef boost::shared_ptr<DecoderWorkerThread> Ptr;

	private:
		void SendNotification(Img::MessageReceiver::LoadMessage msg, Img::Image* image, const String& desc = TX(""));

		static void					threadWrapper(void* param) {
			static_cast<DecoderWorkerThread*>(param)->ThreadMain();
		}

		void						ThreadMain();

		void PerformUpdate();
		void processImage( ImageLoader::Ptr image );
		ImageQueue::Ptr m_queue;
		std::auto_ptr<boost::thread> m_thread;

		enum State {
			StStopped,
			StRunning,
			StUpdating,
			StResuming,
			StTerminating,
		};
		State m_state;

		boost::mutex m_mxWorkList;
		boost::condition m_cdGotWork;
		boost::condition m_cdUpdated;
		boost::condition m_cdRunning;

		bool m_isAlive;

		WorkList m_wl;
		MessageReceiver* m_receiver;
		WorkPackage m_package;
	};
}

#endif
