
#include "StdAfx.h"
#include "main.h"
#include "viewer.h"
#include "assoc_util.h"
#include "setup_wizard.h"
#include "lang_table.inc"
#include "registry.h"
#include "config.h"
#include "illa/codecmgr.h"
#include "assoc_util.h"
#include "osinfo.h"
#include "elevation.h"
#include "orz/intl.h"
#include "registry.h"
#include "control.h"
#include "w32_assure_folder.h"

void SetupAssociationMode() {
	bool isElevated = Win::IsElevated();

	if(Win::GetWindowsVersion() >= Win::WindowsVista) {
		App::Association::SetAssociationsMode(App::Association::AMVista, isElevated);
		if(isElevated) App::Association::SetupMachineLevelRegistry();
	}

	if(Win::GetWindowsVersion() < Win::WindowsVista || App::Association::HasSetupMachineLevelRegistry() == false) {
		App::Association::SetAssociationsMode(App::Association::AMXP, isElevated);
		App::Association::SetupMachineLevelRegistry();
	}

	App::Association::DeleteDeprecatedAssociations();
}
enum WizardStyle {
	WizardNone,
	WizardImplicit,
	WizardExplicit,
	WizardInstall
};
WizardStyle DetermineWizardStyle(const String& params) {
	if (params.substr(0, 21) == TX("--show-wizard-install"))
		return WizardInstall;
	else if (params == TX("--show-wizard"))
		return WizardExplicit;
	else if (Reg::Key(Reg::Keys::DWSetupWizardRun) == 0)
		return WizardImplicit;
	return WizardNone;
}

int start_app(const String& params) {
	Img::CodecFactoryStore cfs;
	cfs.AddBuiltinCodecs();

	try {
		Reg::Load(App::cg_SettingsLocation);

		WizardStyle style = DetermineWizardStyle(params);

		Log.SetOutput(assure_folder((style == WizardNone?App::cg_RunLogLocation:App::cg_SetupLogLocation)));

		Intl::LanguageTable(c_lang_strings, 0, App::FinalSID-1);
		Intl::CurrentLanguage(Intl::Language(Reg::Key(Reg::Keys::DWLanguage)));

		if(style == WizardInstall) {
			COND_STRICT(params.length() > 23, Err::InvalidParam, TX("Language identifier missing."));

			String language = ToLower(params.substr(22, String::npos));
			Intl::Language lang = Intl::LangEnglish;

			if (language == TX("sv"))
				lang = Intl::LangSwedish;
			Log << TX("Installer wizard language: ") << language << TX("\n");

			Intl::CurrentLanguage(lang);
			Log << TX("Language applied.\n");
			Reg::Key(Reg::Keys::DWLanguage, lang);
			Log << TX("Language registered.\n");
		}

		App::Association::SetCodecFactoryStore(&cfs);
		SetupAssociationMode();

		if (params == TX("--cleanup")) {
			App::Association::AssociateNone();
			App::Association::DeleteMachineLevelRegistry();
		}
		else {
			if (style != WizardNone) {
				// No language selection for installer-spawned wizards
				Log << TX("Launching wizard.\n");
				App::Wiz::SetupWizard wiz(&cfs, style != WizardInstall);
				wiz.DoModal(0);
			}

			if (style != WizardInstall) {
				App::Viewer viewer(&cfs, params.c_str());

				// Attempt to display viewer
				if(viewer.Create(0))
					viewer.Show(true);

				viewer.StartApplication();
			}
		}

		Reg::Save(App::cg_SettingsLocation);
	}
	catch(Err::DuplicateInstance&) {
		// This is not something out of the ordinary.
	}
	catch(Err::Exception& e) {
		Log << TX("Exception caught:\n") << e.Desc() << TX("\n");
		MessageBox(0, (TX("An error occurred: ") + e.Desc()).c_str(), TX("Pictus Error"), MB_OK);
		//throw;	// Good for debugging
	}

	Win::Control::DestructControls();

	return EXIT_SUCCESS;
}
