#include "StdAfx.h"
#include "reg_defaultprograms.h"
#include "w32_reg.h"
#include "w32_assure_folder.h"
#include "Config.h"

namespace App {
	const String DefaultPrograms::AppIdBasePath(TX("SOFTWARE\\Poppeman\\Pictus\\"));
	const String DefaultPrograms::RegisteredApplicationsBasePath(TX("SOFTWARE\\RegisteredApplications\\"));

	struct AppRegComObject {
		IApplicationAssociationRegistration* pAAR;
		AppRegComObject():pAAR(0) {
			COND_STRICT(SUCCEEDED(CoCreateInstance(CLSID_ApplicationAssociationRegistration,
				NULL,
				CLSCTX_INPROC,
				__uuidof(IApplicationAssociationRegistration),
				(void**)&pAAR)), Err::CriticalError, TX("Failed to instanciate CLSID_ApplicationAssociationRegistration COM object."));
		}
		~AppRegComObject() {
			if(pAAR)
				pAAR->Release();
		}
	};

	bool DefaultPrograms::IsRegistered() {
		Log << TX("DefaultPrograms::IsRegistered :");
		Win::RegistryKey defBase(Win::RegistryKey::LocalMachine, (AppIdBasePath + TX("Capabilities")));
		bool r = defBase.Open(false, true);
		Log << (r?TX("Yes"):TX("No")) << TX("\n");
		return r;
	}

	void DefaultPrograms::Register(const String& name, const String& description) {
		Log << TX("DefaultPrograms::Register(") << name << TX(", ") << description << TX("\n");
		Win::RegistryKey defBase(Win::RegistryKey::LocalMachine, (AppIdBasePath + TX("Capabilities")));
		defBase.Open(true);
		defBase.StringValue(TX("ApplicationName"), name);
		defBase.StringValue(TX("ApplicationDescription"), description);

		Win::RegistryKey regApps(Win::RegistryKey::LocalMachine, RegisteredApplicationsBasePath);
		regApps.Open(true);
		regApps.StringValue(name, (AppIdBasePath + TX("Capabilities")));
	}

	void DefaultPrograms::AddExtension(const String& progId, const String& ext) {
		Log << TX("DefaultPrograms::AddExtension(") << progId << TX(", ") << ext << TX("\n");
		Win::RegistryKey defBase(Win::RegistryKey::LocalMachine, (AppIdBasePath + TX("Capabilities\\FileAssociations")));
		defBase.Open(true);
		defBase.StringValue(TX(".") + ToLower(ext), progId);
	}

	void DefaultPrograms::Unregister() {
		Win::RegistryKey registeredApps(Win::RegistryKey::LocalMachine, RegisteredApplicationsBasePath);
		if(registeredApps.Open())
			registeredApps.DeleteValue(cg_PictusDefaultProgramsName);

		RecursiveKeyDelete(Win::RegistryKey::LocalMachine, AppIdBasePath);
	}

	void AssociateTo(const String& app, const String& extension) {
		AppRegComObject o;
		Log << TX("DefaultPrograms::AssociateTo(") << app << TX(", ") << extension << TX("): ");
		COND_STRICT(ERROR_SUCCESS == o.pAAR->SetAppAsDefault(app.c_str(), (TX(".")+extension).c_str(), AT_FILEEXTENSION), 
			Err::InvalidParam,
			TX("Could not associate :") + extension);
		Log << TX("Succeeded.\n");
	}

	void DefaultPrograms::Associate(const String& extension) {
		AssociateTo(cg_PictusDefaultProgramsName, extension);
	}

	bool DefaultPrograms::IsAssociated(const String& Extension) {
		AppRegComObject o;
		Log << TX("DefaultPrograms::IsAssociated(") << Extension << TX("): ");

		BOOL b;
		if(S_OK != o.pAAR->QueryAppIsDefault((TX(".")+Extension).c_str(), AT_FILEEXTENSION, AL_EFFECTIVE, cg_PictusDefaultProgramsName, &b)) {
			Log << TX("QueryAppIsDefault failed.\n");
			return false;
		}
		Log << b << TX("\n");
		return b==TRUE;
	}
}
