#include "StdAfx.h"
#include "reg_progid.h"

namespace App {
	String UserClasses = TX("Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\FileExts\\");
	String OldUserClasses = TX("Software\\Classes\\");
	String MachineClasses = TX("SOFTWARE\\Classes\\");
	String AssociationValue = TX("ProgId");

	void ProgId::WriteProgId(FileFormat::Ptr fmt, const String& executable, bool writeInCurrentUser) {
		const String progId(DetermineClassesBase(writeInCurrentUser) + fmt->Identifier());

		Win::RegistryKey progIdBase(DetermineRoot(writeInCurrentUser), progId);
		progIdBase.Open(true);
		progIdBase.StringValue(TX(""), fmt->Description());

		Win::RegistryKey shellCmd(DetermineRoot(writeInCurrentUser), progId + TX("\\shell\\open\\command"));
		shellCmd.Open(true);
		shellCmd.StringValue(TX(""), TX("\"") + executable + TX("\" \"%1\""));

		Win::RegistryKey icon(DetermineRoot(writeInCurrentUser), progId + TX("\\DefaultIcon"));
		icon.Open(true);
		icon.StringValue(TX(""), TX("\"") + executable + TX("\",-") + ToString(fmt->IconId()));
	}

	Win::RegistryKey::Root ProgId::DetermineRoot(bool writeInCurrentUser) {
		if (writeInCurrentUser)
			return Win::RegistryKey::CurrentUser;
		return Win::RegistryKey::LocalMachine;
	}

	String ProgId::DetermineClassesBase(bool writeInCurrentUser) {
		if (writeInCurrentUser)
			return UserClasses;
		return MachineClasses;
	}

	bool ProgId::HasWrittenProgId(FileFormat::Ptr fmt, bool writeInCurrentUser) {
		const String progId(DetermineClassesBase(writeInCurrentUser) + fmt->Identifier());

		Win::RegistryKey progIdBase(DetermineRoot(writeInCurrentUser), progId);
		return progIdBase.Open(false, true);
	}

	bool ProgId::IsAssociated(FileFormat::Ptr fmt) {
		const String& ext = fmt->Extensions()[0];

		Win::RegistryKey key(
			Win::RegistryKey::CurrentUser,
			UserClasses + ToLower((ext[0] == TX('.')) ? ext : (TX(".") + ext)));

		if (key.Open() == false) return false;
		if (key.ValueExist(AssociationValue) && (key.StringValue(AssociationValue) == fmt->Identifier())) return true;
		return false;
	}

	void ProgId::Associate(FileFormat::Ptr fmt) {
		foreach(const String& s, fmt->Extensions()) {
			String keystr = TX(".") + ToLower(s);
			if(HasExtension(fmt->Identifier(), keystr)) continue;

			Win::RegistryKey key(
				Win::RegistryKey::CurrentUser,
				UserClasses + keystr);
			if(key.Open(true) == false) continue;

			// Create backup if one does not already exist
			if (key.ValueExist(TX("oldfiletype_copy")) == false)
				if (key.ValueExist(AssociationValue)) key.StringValue(TX("oldfiletype_copy"), key.StringValue(AssociationValue).c_str());

			key.StringValue(AssociationValue, fmt->Identifier());
		}
	}

	bool ProgId::HasExtension(const String& progId, const String& ext) {
		String keystr = ToLower((ext[0] == TX('.')) ? ext : (TX(".") + ext));

		Win::RegistryKey key(
			Win::RegistryKey::CurrentUser,
			UserClasses + keystr);
		if (key.Open() == false) return false;

		if (key.ValueExist(AssociationValue) && (key.StringValue(AssociationValue) == progId)) return true;
		return false;
	}

	void ProgId::UnAssociate(FileFormat::Ptr fmt) {
		DoDeleteAssociation(fmt, false);
	}


	void ProgId::DeleteDeprecatedAssociation(FileFormat::Ptr fmt) {
		DoDeleteAssociation(fmt, true);
	}


	void ProgId::DoDeleteAssociation(FileFormat::Ptr fmt, bool oldClasses) {
		foreach(const String& s, fmt->Extensions()) {
			String keystr = TX(".") + ToLower(s);
			if (HasExtension(fmt->Identifier(), keystr) == false) continue;

			Win::RegistryKey key(
				Win::RegistryKey::CurrentUser,
				(oldClasses?OldUserClasses:UserClasses) + keystr);

			if (key.Open(false) == false)
				continue;

			String valueName = (oldClasses?TX(""):AssociationValue);

			if (key.ValueExist(TX("oldfiletype_copy")))
				key.StringValue(valueName, key.StringValue(TX("oldfiletype_copy")).c_str());	// Use the backup (if it exists)
			else
				key.DeleteValue(valueName);

			key.DeleteValue(TX("oldfiletype_copy"));
		}
	}


	void ProgId::DeleteProgId(FileFormat::Ptr fmt, bool writeInCurrentUser) {
		const String progId(DetermineClassesBase(writeInCurrentUser) + fmt->Identifier());
		Win::RecursiveKeyDelete(DetermineRoot(writeInCurrentUser), progId);
	}
}
