#include "StdAfx.h"
#include "rendertargetgdi.h"
#include "illa/swsurface.h"
#include "gdi_ddsurface.h"
#include "illa/render.h"
#include "wintypes.h"

namespace Win {
	using namespace Geom;

	Img::Surface::Ptr RendererGDI::CreateSurface() {
		return Img::Surface::Ptr(new Img::SurfaceSoftware);
	}

	Renderer::RenderStatus RendererGDI::OnBeginRender() {
		BeginPaint(TargetWindow(), &m_ps);
		return RS_OK;
	}

	void RendererGDI::OnEndRender() {
		EndPaint(TargetWindow(), &m_ps);
	}

	Geom::SizeInt RendererGDI::RenderAreaSize() {
		RECT cr;
		GetClientRect(TargetWindow(), &cr);
		return Geom::SizeInt(cr.right - cr.left, cr.bottom - cr.top);
	}

	DDSurface::Ptr RendererGDI::OnCreateDDSurface() {
		return DDSurface::Ptr(new DDSurfaceGDI(m_ps.hdc));
	}

	void RendererGDI::OnRenderToDDSurface(DDSurface::Ptr dest, Img::Surface::Ptr source, const Geom::PointInt& zoomedImagePosition, const Geom::RectInt& destinationArea, const Img::Properties& props) {
		COND_STRICT(source, Err::InvalidParam, TX("Source may not be null"));

		COND_STRICT(dest, Err::InvalidParam, TX("Dest may not be null"));
		DDSurfaceGDI* dds = dynamic_cast<DDSurfaceGDI*>(dest.get());
		COND_STRICT(dds, Err::InvalidParam, TX("Dest was not a DDSurfaceGDI"));

		size_t stride = dds->Dimensions().Width * 4;
		Filter::FilterBuffer dst(dds->Dimensions(), 4, dds->GetWritePtr() + stride * destinationArea.Top() + destinationArea.Left() * 4, stride);

		Img::FilterBufferAndLock src = GenerateFilterBuffer(source);

		Img::RenderToBuffer(dst, src.filterBuffer, source->GetFormat(), RectInt(zoomedImagePosition, destinationArea.Dimensions()), props);
	}

	void RendererGDI::OnPresentFromDDSurface(const Geom::RectInt& destRect, DDSurface::Ptr source, const Geom::PointInt& sourceTopLeft) {
		DDSurfaceGDI* dds = dynamic_cast<DDSurfaceGDI*>(source.get());

		Geom::RectInt clippedRect = destRect;
		dds->GetDC().BlitToDC(m_ps.hdc, destRect, sourceTopLeft);
	}

	bool RendererGDI::PanCurrentView(const Geom::SizeInt& offset) {
		return (ScrollWindowEx(TargetWindow(), offset.Width, offset.Height, 0, 0, 0, 0, SW_INVALIDATE) != ERROR);
	}

	Geom::RectInt RendererGDI::GetInvalidArea() {
		return RECTToRect(m_ps.rcPaint);
	}
}
