#include "StdAfx.h"
#include "setup_wizard.h"
#include "res_viewer.h"
#include "registry.h"

#include "wiz_page.h"
#include "wiz_language.h"
#include "wiz_fileformats.h"
#include "wiz_customfileformats.h"

#include "ctrl_button.h"

namespace App {
	namespace Wiz {
		using namespace Intl;

		bool SetupWizard::PerformOnInitDialog() {
			CreateButton(IDOK)->OnClick.connect(bind(&SetupWizard::nextPage, this));
			CreateButton(IDC_BUTTON_INSTALL_NEXT)->OnClick.connect(bind(&SetupWizard::nextPage, this));
			CreateButton(IDCANCEL)->OnClick.connect(bind(&SetupWizard::previousPage, this));
			CreateButton(IDC_BUTTON_INSTALL_PREVIOUS)->OnClick.connect(bind(&SetupWizard::previousPage, this));

			Caption(SIDInstallWindow);
			ControlText(IDC_BUTTON_INSTALL_PREVIOUS, SIDPrevious);
			UpdateButtons();

			for (PageContainer::iterator i = m_pages.begin(); i != m_pages.end(); ++i)
				(*i)->DoModeless(this);

			m_pages[0]->Show(true);
			EnableWindow(GetDlgItem(Handle(), IDC_BUTTON_INSTALL_PREVIOUS), false);

			return true;
		}

		void SetupWizard::UpdateButtons() {
			if ((m_pageIndex + 1) < m_pages.size())
				ControlText(IDC_BUTTON_INSTALL_NEXT, SIDNext);
			else
				ControlText(IDC_BUTTON_INSTALL_NEXT, SIDFinish);
		}

		SetupWizard::SetupWizard(Img::CodecFactoryStore* cfs, bool showLanguageDialog):Win::Dialog(IDD_INSTALL), m_pageIndex(0) {
			if (showLanguageDialog)
				m_pages.push_back(new Language);

			m_pages.push_back(new FileFormats);
			m_pages.push_back(new CustomFileFormats(cfs));
		}

		SetupWizard::~SetupWizard() {
			for (PageContainer::iterator i=m_pages.begin();i!=m_pages.end();++i)
				delete (*i);
		}


		void SetupWizard::previousPage() {
			do {
				m_pages[m_pageIndex]->Show(false);
				if (m_pageIndex>0) --m_pageIndex;
				if (m_pageIndex == 0) {
					EnableWindow(GetDlgItem(Handle(), IDC_BUTTON_INSTALL_PREVIOUS), false);
					SetFocus(GetDlgItem(Handle(), IDC_BUTTON_INSTALL_NEXT));
				}
				else
					EnableWindow(GetDlgItem(Handle(), IDC_BUTTON_INSTALL_PREVIOUS), true);

				m_pages[m_pageIndex]->Show(true);
				UpdateButtons();
			}
			while(m_pages[m_pageIndex]->Hide() == true);
		}

		void SetupWizard::nextPage() {
			// Hide or show next page.
			if ((m_pageIndex + 1) < m_pages.size())
				m_pages[m_pageIndex+1]->Hide(m_pages[m_pageIndex]->HideNext());

			do {
				if ((m_pageIndex+1) == m_pages.size()) {
					writeSettings();
					EndDialog(Handle(), true);
					return;
				}

				m_pages[m_pageIndex]->Show(false);
				m_pageIndex++;
				EnableWindow(GetDlgItem(Handle(), IDC_BUTTON_INSTALL_PREVIOUS), (m_pageIndex!=0?true:false));
				m_pages[m_pageIndex]->Show(true);
				UpdateButtons();
			} while(m_pages[m_pageIndex]->Hide()==true);
		}

		void SetupWizard::writeSettings() {
			// Don't auto-show wizard again
			Reg::Key(Reg::Keys::DWSetupWizardRun, 1);

			for (PageContainer::iterator i = m_pages.begin(); i != m_pages.end(); ++i)
				if ((*i)->Hide() == false) (*i)->writeSettings();
		}
	}
}
