#include "StdAfx.h"
#include "viewer_contextmenu.h"
#include "res_viewer.h"
#include "viewer.h"

namespace App {
	using namespace Win;

	void ViewerContextMenu::Construct(Viewer* v) {
		m_viewer = v;

		m_menu.AddItem(ID_OPEN, SIDOpen);
		Menu::Ptr pZoom(m_menu.AddSubMenu(SIDZoom));
		pZoom->AddItem(ID_ZOOM_FITIMAGE, SIDZoomFitImage);
		pZoom->AddItem(ID_ZOOM_FULLSIZE, SIDZoomFullSize);
		pZoom->AddMenuSeparator();
		pZoom->AddItem(ID_ZOOM_ZOOMIN, SIDMenuZoomIn);
		pZoom->AddItem(ID_ZOOM_ZOOMOUT, SIDMenuZoomOut);

		Menu::Ptr pRot(m_menu.AddSubMenu(SIDOrientation));
		pRot->AddItem(ID_ROTATE_DEFAULT, SIDOrientationDefault);
		pRot->AddItem(ID_ROTATE_FLIPX, SIDOrientationFlipX);
		pRot->AddItem(ID_ROTATE_FLIPY, SIDOrientationFlipY);
		pRot->AddItem(ID_ROTATE_90, SIDOrientation90);
		pRot->AddItem(ID_ROTATE_180, SIDOrientation180);
		pRot->AddItem(ID_ROTATE_270, SIDOrientation270);

		Menu::Ptr pSort(m_menu.AddSubMenu(SIDMenuSort));
		pSort->AddItem(ID_SORT_FILENAME, SIDMenuSortFilename);
		pSort->AddItem(ID_SORT_DATEMODIFIED, SIDMenuSortDateModified);
		pSort->AddItem(ID_SORT_DATEACCESSED, SIDMenuSortDateAccessed);
		pSort->AddItem(ID_SORT_DATECREATED, SIDMenuSortDateCreated);

		Menu::Ptr pWall(m_menu.AddSubMenu(SIDSetWallpaper));
		pWall->AddItem(ID_SETWALLPAPER_STRETCH, SIDWPStretch);
		pWall->AddItem(ID_SETWALLPAPER_CENTER, SIDWPCenter);
		pWall->AddItem(ID_SETWALLPAPER_TILE, SIDWPTile);

		m_menu.AddItem(ID_OPENDIRECTORY, SIDOpenDirectory);
		m_menu.AddItem(ID_ADJUST, SIDAdjust);
		m_menu.AddItem(ID_RENAME, SIDMenuRename);
		m_menu.AddItem(ID_SETTINGS, SIDSettings);

		m_menuMap.AddAction(ID_OPEN, bind(&Viewer::OpenFolder, v));
		m_menuMap.AddAction(ID_ZOOM_ZOOMIN, bind(&Viewer::ZoomIn, v));
		m_menuMap.AddAction(ID_ZOOM_ZOOMOUT, bind(&Viewer::ZoomOut, v));
		m_menuMap.AddAction(ID_ZOOM_FITIMAGE, bind(&Viewer::ZoomMode, v, ZoomFitImage));
		m_menuMap.AddAction(ID_ZOOM_FULLSIZE, bind(&Viewer::ZoomFullSize, v));
		m_menuMap.AddAction(ID_SETTINGS, bind(&Viewer::ShowSettings, v));
		m_menuMap.AddAction(ID_OPENDIRECTORY, bind(&Viewer::OpenDirectoryInExplorer, v));
		m_menuMap.AddAction(ID_ADJUST, bind(&Viewer::ShowAdjust, v));
		m_menuMap.AddAction(ID_RENAME, bind(&Viewer::RenameCurrent, v));
		m_menuMap.AddAction(ID_ROTATE_DEFAULT, bind(&Viewer::Rotate, v, Filter::RotateDefault));
		m_menuMap.AddAction(ID_ROTATE_FLIPX, bind(&Viewer::Rotate, v, Filter::FlipX));
		m_menuMap.AddAction(ID_ROTATE_FLIPY, bind(&Viewer::Rotate, v, Filter::FlipY));
		m_menuMap.AddAction(ID_ROTATE_90, bind(&Viewer::Rotate, v, Filter::Rotate90));
		m_menuMap.AddAction(ID_ROTATE_180, bind(&Viewer::Rotate, v, Filter::Rotate180));
		m_menuMap.AddAction(ID_ROTATE_270, bind(&Viewer::Rotate, v, Filter::Rotate270));
		m_menuMap.AddAction(ID_SORT_FILENAME, bind(&Viewer::Sort, v, Img::Cacher::SortFilename));
		m_menuMap.AddAction(ID_SORT_DATEACCESSED, bind(&Viewer::Sort, v, Img::Cacher::SortDateAccessed));
		m_menuMap.AddAction(ID_SORT_DATECREATED, bind(&Viewer::Sort, v, Img::Cacher::SortDateCreated));
		m_menuMap.AddAction(ID_SORT_DATEMODIFIED, bind(&Viewer::Sort, v, Img::Cacher::SortDateModified));
		m_menuMap.AddAction(ID_SETWALLPAPER_CENTER, bind(&Viewer::ApplyWallpaper, v, Win::Wallpaper::WPCenter));
		m_menuMap.AddAction(ID_SETWALLPAPER_STRETCH, bind(&Viewer::ApplyWallpaper, v, Win::Wallpaper::WPStretch));
		m_menuMap.AddAction(ID_SETWALLPAPER_TILE, bind(&Viewer::ApplyWallpaper, v, Win::Wallpaper::WPTile));

	}

	void ViewerContextMenu::Display(Geom::PointInt pos) {
		m_menuMap.Execute(m_menu.Display(m_viewer, pos));
	}

	void ViewerContextMenu::Zoomed(bool fullSize) {
		m_menu.CheckMenuItem(ID_ZOOM_FITIMAGE, false);
		m_menu.CheckMenuItem(ID_ZOOM_FULLSIZE, fullSize);
	}

	void ViewerContextMenu::FitImage() {
		m_menu.CheckMenuItem(ID_ZOOM_FITIMAGE, true);
		m_menu.CheckMenuItem(ID_ZOOM_FULLSIZE, false);
	}

	ViewerContextMenu::ViewerContextMenu():m_viewer(0) {}
}
