#include "StdAfx.h"
#include "viewer_keyboard.h"
#include "viewer.h"

namespace App {

	void ViewerKeyboard::Construct(Viewer* owner) {
		owner->OnKeyDown.connect(bind(&KeyActionMap::Execute, &m_keyMap, _1));

		AddMapping(VK_F2, bind(&Viewer::RenameCurrent, owner));
		AddMapping(VK_ESCAPE, boost::bind(&Viewer::Close, owner));

		KeyActionMap::Function_Type ZoomIn = bind(&Viewer::ZoomIn, owner);
		AddMapping(VK_OEM_PLUS, ZoomIn);
		AddMapping(VK_ADD, ZoomIn);
		AddMapping(VK_NUMPAD0, ZoomIn);

		KeyActionMap::Function_Type ZoomOut = bind(&Viewer::ZoomOut, owner);
		AddMapping(VK_OEM_MINUS, ZoomOut);
		AddMapping(VK_SUBTRACT, ZoomOut);
		AddMapping(VK_NUMPAD1, ZoomOut);

		AddMapping(VK_MULTIPLY, bind(&Viewer::ZoomMode, owner, ZoomFree));
		AddMapping(VK_UP, bind(&Viewer::PanUp, owner));
		AddMapping(VK_LEFT, bind(&Viewer::PanLeft, owner));
		AddMapping(VK_DOWN, bind(&Viewer::PanDown, owner));
		AddMapping(VK_RIGHT, bind(&Viewer::PanRight, owner));
		AddMapping(VK_HOME, bind(&Viewer::ToStart, owner));
		AddMapping(VK_END, bind(&Viewer::ToEnd, owner));

		KeyActionMap::Function_Type ImageNext = bind(&Viewer::ImageNext, owner, 1);
		KeyActionMap::Function_Type ImageNextSkip = bind(&Viewer::ImageNext, owner, Viewer::ShiftSkipLength);
		AddMapping(VK_SPACE, ImageNext);
		AddMapping(VK_NEXT, ImageNext);
		AddMapping(VK_SPACE, ImageNextSkip, Shift);
		AddMapping(VK_NEXT, ImageNextSkip, Shift);
		AddMapping(VK_LEFT, ImageNext, Alt);

		KeyActionMap::Function_Type ImagePrev = bind(&Viewer::ImagePrev, owner, 1);
		KeyActionMap::Function_Type ImagePrevSkip = bind(&Viewer::ImagePrev, owner, Viewer::ShiftSkipLength);
		AddMapping(VK_BACK, ImagePrev);
		AddMapping(VK_PRIOR, ImagePrev);
		AddMapping(VK_BACK, ImagePrevSkip, Shift);
		AddMapping(VK_PRIOR, ImagePrevSkip, Shift);
		AddMapping(VK_RIGHT, ImagePrev, Alt);


		AddMapping(VK_DELETE, bind(&Viewer::RemoveImage, owner, Viewer::RemoveRecycle));
		AddMapping(VK_DELETE, bind(&Viewer::RemoveImage, owner, Viewer::RemoveDelete), Shift);
		AddMapping(VK_DELETE, bind(&Viewer::RemoveImage, owner, Viewer::RemoveOnly), Ctrl);

		KeyActionMap::Function_Type ImageRandom = bind(&Viewer::ImageRandom, owner);
		AddMapping('R', ImageRandom);
		AddMapping('r', ImageRandom);

		KeyActionMap::Function_Type ShowSettings = bind(&Viewer::ShowSettings, owner);
		AddMapping('O', ShowSettings);
		AddMapping('o', ShowSettings);

		AddMapping(VK_RETURN, bind(&Viewer::ToggleFullscreenMode, owner), Alt);
	}

	void ViewerKeyboard::AddMapping(WPARAM key, KeyActionMap::Function_Type f, int flags) {
		m_keyMap.AddAction(Win::KeyEvent(
			key,
			(flags & Alt) != 0,
			(flags & Ctrl) != 0,
			(flags & Shift) != 0),
			f);
	}
}
