#ifndef W32_REG_H
#define W32_REG_H

#include "app_types.h"

namespace Err {
	class Win32RegistryError:public Exception {
	public:
		Win32RegistryError(const String& msg):Exception(TX("Win32 registry: ")+msg) {}
	};
}

namespace Win {
	class RegistryKey {
	public:
		enum Root {
			ClassesRoot,
			CurrentUser,
			LocalMachine,
			Users,
			CurrentConfig
		};

		bool Open(bool doCreate = false, bool readOnly = false);
		void Create();
		bool Exist();
		bool ValueExist(const String& name);
		void DeleteValue(const String& name);
		void Close();

		std::vector<Byte> BinaryValue(const String& name);

		String StringValue(const String& name);
		void StringValue(const String& name, const String& value);

		DWord DWORDValue(const String& name);
		void DWORDValue(const String& name, const DWord& value);

		RegistryKey(const Root root, const String& name);
		~RegistryKey();

	private:
		std::vector<Byte> GetValue(const String& name, DWORD type);

		HKEY m_key;
		HKEY m_root;
		String m_name;
	};

	HKEY DetermineRootHKEY(RegistryKey::Root r);
	bool RecursiveKeyDelete(const RegistryKey::Root root, const String& name);

};

#endif
