#include "StdAfx.h"
#include "w32_rendertarget.h"
#include "illa/config.h"

namespace Win {
	void WinRenderTarget::SetRenderer( Renderer::Ptr newRenderer ) {
		m_renderer = newRenderer;
	}

	void WinRenderTarget::SetRedrawStrategy( RedrawStrategy::Ptr newRedrawStrategy ) {
		m_redrawStrategy = newRedrawStrategy;
	}

	WinRenderTarget::WinRenderTarget()
		:m_hwnd(0)/*,
		 m_filterMagnification(Filter::FilterUndefined),
		 m_filterMinification(Filter::FilterUndefined),
		 m_colorBackground(0xff, 0xff, 0xff, 0xff),
		 m_brightness(0),
		 m_gamma(10),
		 m_contrast(Img::ContrastStep),
		 m_rotation(Filter::RotateDefault)*/
	{}

	bool WinRenderTarget::TargetWindow( HWND hwnd ) {
		if(m_renderer == 0)
			return false;

		m_hwnd = hwnd;
		return m_renderer->TargetWindow(hwnd);
	}

	void WinRenderTarget::Render( const Geom::PointInt& sourceZoomedTopLeft, const Img::Properties& props ) {
		COND_WARNING(m_hwnd, Err::CriticalError, TX("TargetWindow not set"));
		COND_WARNING(m_renderer, Err::CriticalError, TX("Renderer not set"));
		COND_WARNING(m_redrawStrategy, Err::CriticalError, TX("Redraw strategy not set"));

		m_redrawStrategy->Render(m_renderer, m_surface, sourceZoomedTopLeft, props);
	}

	void WinRenderTarget::SetSurface( Img::Surface::Ptr surface ) {
		m_surface = surface;
	}

	Img::Surface::Ptr WinRenderTarget::CurrentSurface() const {
		return m_surface;
	}

	/*Filter::FilterMode WinRenderTarget::ActiveFilterMode() const {
		Filter::FilterMode toReturn = Filter::FilterUndefined;
		if (Zoom() < 1.0)
			toReturn = m_filterMinification;
		else
			toReturn = m_filterMagnification;

		COND_STRICT(toReturn != Filter::FilterUndefined, Err::CriticalError, TX("Filter mode not set"));
		return toReturn;
	}

	void WinRenderTarget::MagnificationFilter( Filter::FilterMode mode ) {
		COND_STRICT(mode > 0 && mode < Filter::FilterNum, Err::InvalidParam, TX("Invalid filtermode: ") + ToString(mode));
		m_filterMagnification =  mode;
	}

	void WinRenderTarget::MinificationFilter( Filter::FilterMode mode ) {
		COND_STRICT(mode > 0 && mode < Filter::FilterNum, Err::InvalidParam, TX("Invalid filtermode: ") + ToString(mode));
		m_filterMinification = mode;
	}

	int WinRenderTarget::Brightness() const {
		return m_brightness;
	}

	void WinRenderTarget::Brightness( int brightness ) {
		COND_WARNING(brightness >= Img::MinBrightness, Err::InvalidParam, TX("Brightness too low"));
		COND_WARNING(brightness <= Img::MaxBrightness, Err::InvalidParam, TX("Brightness too high"));

		m_brightness =  brightness;
	}

	int WinRenderTarget::Contrast() const {
		return m_contrast;
	}

	void WinRenderTarget::Contrast( int contrast ) {
		COND_WARNING(contrast >= Img::MinContrast, Err::InvalidParam, TX("Contrast too low"));
		COND_WARNING(contrast <= Img::MaxContrast, Err::InvalidParam, TX("Contrast too high"));

		m_contrast = contrast;
	}

	int WinRenderTarget::Gamma() const {
		return m_gamma;
	}

	void WinRenderTarget::Gamma( int gamma ) {
		COND_WARNING(gamma >= Img::MinGamma, Err::InvalidParam, TX("Gamma too low"));
		COND_WARNING(gamma <= Img::MaxGamma, Err::InvalidParam, TX("Gamma too high"));

		m_gamma = gamma;
	}

	void WinRenderTarget::BackgroundColor( const Img::Color& color ) {
		m_colorBackground = color;
	}

	const Img::Color& WinRenderTarget::BackgroundColor() const {
		return m_colorBackground;
	}

	void WinRenderTarget::Rotate( Filter::RotationAngle newRotation ) {
		m_rotation =  newRotation;
	}

	Filter::RotationAngle WinRenderTarget::Rotation() const {
		return m_rotation;
	}

	Geom::SizeInt WinRenderTarget::CurrentSurfaceSizeUnzoomed() const {
		Img::Surface::Ptr currentSurface(CurrentSurface());

		if (currentSurface == 0) {
			return Geom::SizeInt(0, 0);
		}

		Geom::SizeInt size = currentSurface->GetSize();

		if ((Rotation() == Filter::Rotate90) || (Rotation() == Filter::Rotate270))
			size = Geom::SizeInt(size.Height, size.Width);
		return size;
	}*/

	Img::Surface::Ptr WinRenderTarget::CreateSurface() {
		COND_WARNING(m_renderer, Err::CriticalError, TX("Renderer not yet set."));
		return m_renderer->CreateSurface();
	}
}
