#include "StdAfx.h"

#include "BasicThread.h"
#include "Logger.h"
#include "main.h"

size_t BasicThread::m_idThreadCounter = 0;

void BasicThread::Terminate()
{
	Output(TX("Terminating ... "));
	boost::mutex::scoped_lock l(m_mutexTerm);
	m_isTerminating = true;
	l.unlock();
	m_thread->join();
}

BasicThread::BasicThread()
	:m_id(m_idThreadCounter++),
	 m_isTerminating(false)
{}

BasicThread::~BasicThread() {}

bool BasicThread::IsTerminating()
{
	boost::mutex::scoped_lock l(m_mutexTerm);
	return m_isTerminating;
}

void BasicThread::Output(const String& s)
{
	OutputString(ToString(m_id) + TX(" - ") + s);
}

void BasicThread::threadWrapper(void* param)
{
	BasicThread* currentThread = static_cast<BasicThread*>(param);
	if (currentThread == 0)
	{
		OutputString(TX("Invalid parameter to thread function!"));
		return;
	}
	currentThread->Output(TX("Thread started, entering ThreadMain..."));
	try {
		currentThread->ThreadMain();
	}
	catch(Err::Exception& e) {
		SignalError(TX("Thread: ") + ToString(currentThread->m_id) + TX("\n") + e.Desc());
	}
	catch(std::exception& e) {
		SignalError(TX("Thread: ") + ToString(currentThread->m_id) + TX("\n") + ToString(e.what()));
	}
	catch(...) {
		SignalError(TX("Thread: ") + ToString(currentThread->m_id) + TX("\n") + TX("Unknown exception type"));
	}
	currentThread->Output(TX("Thread is terminating ..."));
}

void BasicThread::ThreadMain()
{}

void BasicThread::Run()
{
	boost::mutex::scoped_lock l(m_mutexTerm);

	m_isTerminating = false;
	Output(TX("Starting thread ... "));
	m_thread = Util::ThreadPtr(new boost::thread(Util::ThreadAdapter(BasicThread::threadWrapper, this)));
	Output(TX("Thread started!"));
}
