#include "StdAfx.h"

#include "d3d_vertexbuffer.h"
#include "d3d_common.h"

namespace D3D {
	VertexBuffer::VertexBuffer( LPDIRECT3DVERTEXBUFFER9 vb, DWORD fvf, size_t bytes ):m_vb(vb), m_isLocked(false), m_fvf(fvf), m_sizeInBytes(bytes) {
		COND_STRICT(vb, Err::CriticalError, TX("Vertex buffer not initialized"));
	}

	VertexBuffer::~VertexBuffer() {
		SAFE_RELEASE(m_vb);
	}

	Byte* VertexBuffer::Lock() {
		COND_STRICT(m_isLocked == false, Err::CriticalError, TX("Vertex buffer is already locked."));
		void* buf;
		m_vb->Lock(0, 0, &buf, 0);
		m_isLocked = true;
		return reinterpret_cast<Byte*>(buf);
	}

	void VertexBuffer::Unlock() {
		COND_STRICT(m_isLocked, Err::CriticalError, TX("Vertex buffer is not locked."));
		m_vb->Unlock();
		m_isLocked = false;
	}

	size_t VertexBuffer::SizeInBytes() const {
		return m_sizeInBytes;
	}

	LPDIRECT3DVERTEXBUFFER9 VertexBuffer::D3DObject() {
		return m_vb;
	}

	DWORD VertexBuffer::FVFDefinition() {
		return m_fvf;
	}
}
