#ifndef C_PSP_H
#define C_PSP_H

#include "orz/file_cache.h"

#include "codec_static.h"
#include "psp_defines.h"

namespace Img {
	class CodecPSP:public StaticCodec {
	public:
		enum {
			ScansPerChunk = 16
		};

		CodecPSP();

	private:
		bool PerformLoadHeader(IO::FileReader::Ptr file, ImageInfo& info);
		AllocationStatus PerformAllocate();
		LoadStatus PerformLoadImageData(IO::FileReader::Ptr file);

		static const Byte FileHeader[32];
		static const Byte Gen_HeaderID[4];

		struct BlockHeader {
			PSPBlockID ID;
			DWord Length;

			FileInt PositionStart;
			FileInt PositionEnd;
		};

		struct Layer {
			// String Name;
			PSPLayerType Type;
			//Geom::RectInt LayerArea;
			Geom::RectInt SavedLayerArea;
			//Byte					Opacity;
			PSPBlendModes BlendMode;
			Byte LayerFlags;
			// Byte TransparencyProtected;
			// Byte LinkGroup;
			// Geom::RectInt MaskArea;
			// Geom::RectInt SavedMaskArea;
			// Byte IsMaskLinked;
			// Byte IsMaskDisabled;
			// Byte InvertMaskOnBlend;
			// Word BlendRangeCount;
			// Byte SourceBlendRange1[4];
			// Byte DestinationBlendRange1[4];
			// Byte SourceBlendRange2[4];
			// Byte DestinationBlendRange2[4];
			// Byte SourceBlendRange3[4];
			// Byte DestinationBlendRange3[4];
			// Byte SourceBlendRange4[4];
			// Byte DestinationBlendRange4[4];
			// Byte SourceBlendRange5[4];
			// Byte DestinationBlendRange5[4];
			// Byte UseHighlightColor;
			// DWord HighlightColor;
		};

		struct Channel {
			DWord SizeCompressed;
			DWord SizeUncompressed;
			PSPDIBType TypeDIB;
			PSPChannelType TypeChannel;
			FileInt PositionData;

			FileInt CurrentSourcePosition;
		};

		bool ReadBlockHeader(IO::FileReader::Ptr reader, BlockHeader* outHeader);

		bool ReadLayerBlock(IO::FileReader::Ptr reader, Layer* outLayer);
		bool ReadChannelBlock(IO::FileReader::Ptr reader, Channel* outChannel);

		void ParseFileStructure(IO::FileReader::Ptr reader);

		bool DecodeScanlines(IO::FileReader::Ptr reader, int scansToProcess, int width, size_t stride, size_t pixelSize, Byte* destination);

		bool DecodeRawScanlines(IO::FileReader::Ptr reader, int scansToProcess, int width, size_t stride, size_t pixelSize, Byte* destination);
		bool DecodeRLEScanlines(IO::FileReader::Ptr reader, int scansToProcess, int width, size_t stride, size_t pixelSize, Byte* destination);

		bool DecodeRLEChannel(IO::FileReader::Ptr reader, int scansToProcess, int width, size_t stride, size_t pixelSize, Byte* destination);

		Layer m_layer;

		Byte m_bitDepth;

		typedef std::vector<Channel> ChannelVector;

		ChannelVector m_channels;

		PSPCompression m_compression;
		bool m_isStructureParsed;

		int m_currScanline;

		IO::FileReaderByteStreamer m_fileStream;
	};
}

#endif
