#include "StdAfx.h"
#include "c_tga_header.h"

namespace Img {
	bool TGAHeader::LoadHeader(IO::FileReader::Ptr file) {
		if (file->Size() - file->Position() < 19) return false;

		file->Read(&IdSize, 1, 1);
		file->Read(&ColorMapType, 1, 1);
		file->Read(&ImageType, 1, 1);
		file->Read(&ColorMapStart, 2, 1);
		file->Read(&ColorMapLength, 2, 1);
		file->Read(&ColorMapBits, 1, 1);

		// Skip X/Y start coordinates
		file->Seek(4, IO::SeekMethodCurrent);
		file->Read(&Size.Width, 2, 1);
		file->Read(&Size.Height, 2, 1);
		file->Read(&ColorDepth, 1, 1);

		// 00vhaaaa (vertical flip, horizontal flip, alpha bits)
		Byte desc;
		file->Read(&desc, 1, 1);

		// Parse the desc-byte
		AlphaBits		= desc & 0x0f;
		FlipHorizontal	= ((desc & 0x10) != 0);
		FlipVertical	= ((desc & 0x20) == 0);

		if (AlphaBits)
			AttributesType = AT_Alpha;

		FileInt headerEnd = file->Position();

		file->Seek(file->Size() - 26, IO::SeekMethodBegin);
		AChar footer[26];
		file->Read(footer, 26, 1);
		if (memcmp(footer + 8, "TRUEVISION-XFILE", 16) == 0) {
			AttributesType = AT_UndefinedIgnore;

			// New TGA format. See if there's an extension area
			file->Seek(file->Size() - 26, IO::SeekMethodBegin);
			DWord offset = 0;
			file->Read(&offset, 4, 1);
			if (offset != 0) {
				file->Seek(offset, IO::SeekMethodBegin);
				if (!ReadExtensionArea(file)) return false;
			}
		}

		file->Seek(headerEnd, IO::SeekMethodBegin);

		return true;
	}

	bool TGAHeader::ReadExtensionArea( IO::FileReader::Ptr file ) {
		Word size;
		file->Read(&size, 2, 1);
		if (size < 495) return false;

		// Skip tons of worthless data
		file->Seek(492, IO::SeekMethodCurrent);
		AttributesType = AT_NoAlpha;
		file->Read(&AttributesType, 1, 1);
		
		// HACK: Some software writes NoAlpha even though alpha-bits are present
		if(AttributesType == AT_NoAlpha && AlphaBits)
			AttributesType = AT_Alpha;

		return true;
	}

	Img::Format TGAHeader::SurfaceFormat() {
		if (ColorDepth == 8)
			return Img::FormatIndex8;

		if (ColorDepth == 16) {
			if (AttributesType == AT_Alpha)
				return Img::FormatARGB1555;
			else
				return Img::FormatXRGB1555;
		}

		if (ColorDepth == 24)
			return Img::FormatXRGB8888;

		if (ColorDepth == 32) {
			if (AttributesType == AT_Alpha)
				return Img::FormatARGB8888;
			else
				return Img::FormatXRGB8888;
		}

		return Img::FormatUndefined;
	}

	TGAHeader::TGAHeader()
		:IdSize(0),
		ColorMapType(0),
		ImageType(NoData),
		ColorMapStart(0),
		ColorMapLength(0),
		ColorMapBits(0),
		Size(0, 0),
		ColorDepth(0),
		AlphaBits(0),
		FlipHorizontal(false),
		FlipVertical(false)
	{}
}
