#include "StdAfx.h"

#include "c_wbmp.h"
#include "orz/bitconvert.h"

namespace Img {
	bool CodecWBMP::PerformLoadHeader( IO::FileReader::Ptr file, ImageInfo& info ) {
		// The standard only covers type 0 images, so no need to be anymore advanced here
		if (ReadUIntVar(file) != 0) return false;

		Byte sign;
		file->ReadFull(&sign, 1);
		if (sign != 0) return false;

		info.Dimensions.Width = ReadUIntVar(file);
		info.Dimensions.Height = ReadUIntVar(file);
		info.SurfaceFormat = Img::FormatIndex8;

		m_currScan = 0;

		return true;
	}

	AbstractCodec::AllocationStatus CodecWBMP::PerformAllocate() {
		GetSurface()->CreateSurface(GetSize(), GetFormat());
		GetSurface()->ClearSurface(Img::Color(0xff, 0xff, 0xff, 0xff));

		GetSurface()->SetPalette(Img::Grayscale(0, 1));
		return AllocOk;
	}

	AbstractCodec::LoadStatus CodecWBMP::PerformLoadImageData( IO::FileReader::Ptr file ) {
		using namespace Geom;

		typedef boost::shared_array<Byte> ByteArray;

		int width = (GetSize().Width);
		int scanBytes = (width + 7) / 8;
		ByteArray currScan(new Byte[scanBytes]);

		while (DoTerminate() == false) {
			if (m_currScan == GetSize().Height)
				return LoadFinished;

			Surface::LockedArea::Ptr area = GetSurface()->LockSurface(RectInt(PointInt(0, m_currScan), SizeInt(width, 1)));

			file->ReadFull(currScan.get(), scanBytes);
			Util::BitsToBytes(area->Buffer(), currScan.get(), width);
			m_currScan++;
		}
		return LoadAborted;
	}

	DWord CodecWBMP::ReadUIntVar( IO::FileReader::Ptr file ) {
		Byte b;
		DWord currentValue = 0;

		do {
			file->ReadFull(&b, 1);
			currentValue = (currentValue << 7) + (b & 0x7F);
		} while((b & 0x80) != 0);

		return currentValue;
	}
}
