#ifndef CODECMGR_H
#define CODECMGR_H

#include "types.h"

namespace Img {
	class AbstractCodec;
	class ICodecFactory;

	class CodecFactoryStore {
	public:
		struct Info {
			typedef std::vector<String>	ExtensionVector;

			String					Description;
			ExtensionVector			Extensions;
		};
		typedef std::vector<Info> InfoVector;
		bool DoCodecExist(const Char* ext);
		AbstractCodec* CreateCodec(const String& ext);
		AbstractCodec* CreateCodec(size_t index);
		const InfoVector& CodecInfo();

		bool AddCodecFactory(ICodecFactory* pCodecFactory);
		void ReleaseFactories();
		void AddBuiltinCodecs();

		CodecFactoryStore();

	private:
		typedef boost::shared_ptr<ICodecFactory>	FactoryPtr;
		typedef std::deque<FactoryPtr>				FactoryList;
		typedef std::map<String, FactoryPtr>		ExtMap;
		typedef std::pair<String, FactoryPtr>		ExtPair;
		FactoryList				m_factories;
		ExtMap					m_ext;
		InfoVector				m_info;
	};
}

#endif
