#ifndef COLOR_CONV_H
#define COLOR_CONV_H

#include "types.h"

namespace Img {
	inline DWord ToARGBDWORD(int A, int R, int G, int B)	{	return (A << 24) + (R << 16) + (G << 8) + B;	}
	inline DWord ToARGBDWORD(const Img::Color& color)		{	return (color.A << 24) + (color.R << 16) + (color.G << 8) + color.B;	}
	inline Word ToRGBWord565(const Img::Color& color)		{	return ((color.R & 0xf8) << 11) | ((color.G & 0xfc) << 5) | (color.B & 0xf8);	}
	inline Word ToARGBWord1555(const Img::Color& color)		{	return ((color.A & 0x80) << 15) | ((color.R & 0xf8) << 10) | ((color.G & 0xf8) << 5) | (color.B & 0xf8);	}
	inline Byte ChannelARGB8888Alpha(const DWord& val)		{	return static_cast<Byte>(val >> 24);	}
	inline Byte ChannelARGB8888Red(const DWord& val)		{	return static_cast<Byte>((val & 0xff0000) >> 16);	}
	inline Byte ChannelARGB8888Green(const DWord& val)		{	return static_cast<Byte>((val & 0xff00) >> 8);	}
	inline Byte ChannelARGB8888Blue(const DWord& val)		{	return static_cast<Byte>(val & 0xff);	}
	inline Byte ChannelARGB1555Alpha(const Word& val)		{	return static_cast<Byte>(val >> 15);	}
	inline Byte ChannelARGB1555Red(const Word& val)			{	return static_cast<Byte>((val & 0x7C00) >> 10);	}
	inline Byte ChannelARGB1555Green(const Word& val)		{	return static_cast<Byte>((val & 0x03E0) >> 5);	}
	inline Byte ChannelARGB1555Blue(const Word& val)		{	return static_cast<Byte>(val & 0x1f);	}
	inline Byte ChannelRGB565Red(const Word& val)			{	return static_cast<Byte>((val & 0xF8C0) >> 11);	}
	inline Byte ChannelRGB565Green(const Word& val)			{	return static_cast<Byte>((val & 0x07E0) >> 5);	}
	inline Byte ChannelRGB565Blue(const Word& val)			{	return static_cast<Byte>(val & 0x1f);	}
	inline Byte Channel5To8(const Byte& val)				{	return (val << 3) | (val >> 2);	}
	inline Byte Channel6To8(const Byte& val)				{	return (val << 2) | (val >> 4);	}

	inline DWord ABGR_ARGB(const DWord& v) { return ToARGBDWORD(ChannelARGB8888Alpha(v), ChannelARGB8888Blue(v), ChannelARGB8888Green(v), ChannelARGB8888Red(v)); }

	inline DWord ConvertARGB1555ToARGB8888(const Word& src) {
		return Img::ToARGBDWORD(
			(ChannelARGB1555Alpha(src) * 255),
			Channel5To8(ChannelARGB1555Red(src)),
			Channel5To8(ChannelARGB1555Green(src)),
			Channel5To8(ChannelARGB1555Blue(src)));
	}

	inline DWord ConvertXRGB1555ToARGB8888(const Word& src) {
		return Img::ToARGBDWORD(
			0xff,
			Channel5To8(ChannelARGB1555Red(src)),
			Channel5To8(ChannelARGB1555Green(src)),
			Channel5To8(ChannelARGB1555Blue(src)));
	}

	inline DWord ConvertRGB565ToARGB8888(const Word& src) {
		return Img::ToARGBDWORD(
			0xff,
			Channel5To8(ChannelRGB565Red(src)),
			Channel6To8(ChannelRGB565Green(src)),
			Channel5To8(ChannelRGB565Blue(src)));	}

	inline DWord MixARGB8888ARGB8888(const DWord& pix1, const DWord& pix2, Byte alpha) {
		int ia = alpha + ((alpha & 0x80) >> 7);
		DWord dstrb = pix1      & 0xFF00FF;
		DWord dstag = pix1 >> 8 & 0xFF00FF;

		DWord srcrb = pix2      & 0xFF00FF;
		DWord srcag = pix2 >> 8 & 0xFF00FF;

		DWord drb = srcrb - dstrb;
		DWord dag = srcag - dstag;

		drb *= (ia);
		dag *= (ia);
		drb >>= 8;
		dag >>= 8;

		const DWord rb  = (drb + dstrb)      & 0x00FF00FF;
		const DWord ag  = (dag + dstag) << 8 & 0xFF00FF00;

		return rb | ag;
	}
}

#endif
