#include "StdAfx.h"

#include "filter.h"
#include "filter_int.h"
#include "filterjob.h"
#include "orz/threadpool.h"

namespace Filter {
	using namespace Geom;
	using namespace Internal;

	static const int MinimumChunkWidth = 256;

	void Scale::AutoFilter(const FilterMode mode, const FilterBuffer& source, FilterBuffer& dest, const Geom::RectInt& uncroppedRegion, Img::Format format, float zoom) {
		COND_STRICT(mode >= 0 && mode < FilterNum, Err::InvalidParam, TX("Invalid filter mode."));
		COND_STRICT(source.BufferData != 0 && dest.BufferData != 0, Err::InvalidParam, TX("Null buffer not allowed."));
		COND_STRICT((uncroppedRegion.Top() >= 0) && (uncroppedRegion.Left() >= 0), Err::InvalidParam, TX("Negative coordinates in region."));
		Geom::RectInt region = CropResampleRegions(uncroppedRegion, source.Dimensions, dest.Dimensions, zoom);
		COND_STRICT(IsPositive(region.Dimensions()), Err::InvalidParam, TX("Invalid region."));

		boost::shared_ptr<Util::Threadpool> pool;
		if(!pool)
			pool.reset(new Util::Threadpool(4));

		FilterJob::SetSource(mode, &source, format, zoom);

		if (zoom == 1.0f)
			return DirectCopy(source, dest, region, format);

		if(region.Width() > MinimumChunkWidth && region.Height() > FilterJob::ChunkSize) {
			std::vector<shared_ptr<FilterJob> > jobs;

			int num_segments = region.Height() / FilterJob::ChunkSize + 1;
			int num_prior = jobs.size();
			int num_diff = num_segments - num_prior;
			jobs.resize(num_segments);
			for(int i = 0; i < num_diff; ++i)
				jobs[i+num_prior].reset(new FilterJob());

			for(int i = 0; i < num_segments; ++i) {
				jobs[i]->Construct(i, dest, region);
				pool->AddTask(bind(&FilterJob::Run, jobs[i]));
			}
			pool->JoinAll();
			return;
		}

		FilterJob f;
		f.Construct(-1, dest, region);
		f.Run();
	}
}
