#include "StdAfx.h"
#include "filter.h"

namespace Filter {
	void Alpha::IndexedColorKey( const FilterBuffer& source, FilterBuffer& dest, const Geom::RectInt& sourceRegion, const Geom::PointInt& destPos, Img::Format sourceFormat, Byte colorKeyIndex ) {
		COND_STRICT(source.BufferData != 0 && dest.BufferData != 0, Err::InvalidParam, TX("Null buffer not allowed."));
		COND_STRICT(sourceFormat == Img::FormatIndex8, Err::InvalidParam, TX("Surface format must be indexed."));
		//COND_STRICT(source.Palette, Err::InvalidParam, TX("Palette not set."));

		int y1 = Util::Max(0, sourceRegion.Top());
		int y2 = Util::Min(source.Dimensions.Height, sourceRegion.Bottom());
		int x1 = Util::Max(0, sourceRegion.Left());
		int x2 = Util::Min(source.Dimensions.Width, sourceRegion.Right());

		if ((x2 + destPos.X) > dest.Dimensions.Width)
			x2 = dest.Dimensions.Width - destPos.X;

		if ((y2 + destPos.Y) > dest.Dimensions.Height)
			y2 = dest.Dimensions.Height - destPos.Y;

		Byte* sourceCurrentScanlinePtr = source.BufferData;
		Byte* destCurrentScanlinePtr = dest.BufferData + destPos.X * 4 + destPos.Y * dest.Stride;

		for (int y = y1; y < y2; ++y) {
			Byte* sourceCurrentPixelPtr	= reinterpret_cast<Byte*>(sourceCurrentScanlinePtr);
			DWord* destCurrentPixelPtr	= reinterpret_cast<DWord*>(destCurrentScanlinePtr);

			for(int x = x1; x < x2; ++x) {
				Byte sp = *(sourceCurrentPixelPtr++);
				if (sp != colorKeyIndex)
					*destCurrentPixelPtr = Img::ToARGBDWORD(source.Palette.Color(sp));

				destCurrentPixelPtr++;
			}
			sourceCurrentScanlinePtr += source.Stride;
			destCurrentScanlinePtr += dest.Stride;
		}
	}
}
