#include "StdAfx.h"

#include "filter_int.h"

namespace Filter {
	namespace Scale {
		void DirectCopy(const FilterBuffer& source, FilterBuffer& dest, const Geom::RectInt& region, Img::Format format) {
			int pixelSize = Img::EstimatePixelSize(format);

			int width = region.Width();
			int scans = region.Height();

			size_t srcstride = source.Stride;
			size_t dststride = dest.Stride;

			if ((format == Img::FormatXRGB8888) || (format == Img::FormatARGB8888)) {
				ptrdiff_t srcPos	= region.Top() * srcstride + region.Left() * pixelSize;
				ptrdiff_t dstPos	= 0;

				for(int i = 0; i < scans; ++i) {
					memcpy(dest.BufferData + dstPos, source.BufferData + srcPos, width * pixelSize);
					srcPos += srcstride;
					dstPos += dststride;
				}
			}
			else if ((format == Img::FormatARGB1555) || (format == Img::FormatXRGB1555)) {
				ptrdiff_t srcPos	= region.Top() * (srcstride / 2) + region.Left();
				ptrdiff_t dstPos	= 0;

				DWord*	pDst = reinterpret_cast<DWord*>(dest.BufferData);
				Word*	pSrc = reinterpret_cast<Word*>(source.BufferData);

				ptrdiff_t sldiff = srcstride / 2 - width;
				ptrdiff_t dldiff = dststride / 4 - width;

				for(int i = 0; i < scans; ++i) {
					for(int x = 0; x < width; ++x) {
						pDst[dstPos] = Img::ConvertARGB1555ToARGB8888(pSrc[srcPos]);
						srcPos++;
						dstPos++;
					}
					srcPos += sldiff;
					dstPos += dldiff;
				}
			}
			else if (format == Img::FormatRGB565) {
				ptrdiff_t srcPos	= region.Top() * (srcstride / 2) + region.Left();
				ptrdiff_t dstPos	= 0;

				DWord*	pDst = reinterpret_cast<DWord*>(dest.BufferData);
				Word*	pSrc = reinterpret_cast<Word*>(source.BufferData);

				ptrdiff_t sldiff	= srcstride / 2 - width;
				ptrdiff_t dldiff	= dststride / 4 - width;

				for(int i = 0; i < scans; ++i) {
					for(int x = 0; x < width; ++x) {
						pDst[dstPos] = Img::ConvertRGB565ToARGB8888(pSrc[srcPos]);
						srcPos++;
						dstPos++;
					}
					srcPos += sldiff;
					dstPos += dldiff;
				}
			}
			else if (format == Img::FormatIndex8) {
				ptrdiff_t srcPos	= region.Top() * srcstride + region.Left();
				ptrdiff_t dstPos	= 0;

				DWord*	pDst = reinterpret_cast<DWord*>(dest.BufferData);
				Byte*	pSrc = reinterpret_cast<Byte*>(source.BufferData);

				ptrdiff_t sldiff	= srcstride - width;
				ptrdiff_t dldiff	= dststride / 4 - width;

				for(int i = 0; i < scans; ++i) {
					for(int x = 0; x < width; ++x) {
						pDst[dstPos++] = Img::ToARGBDWORD(source.Palette.Color(pSrc[srcPos]));
						srcPos++;
					}
					srcPos += sldiff;
					dstPos += dldiff;
				}
			}
			else
				DO_THROW(Err::InvalidParam, TX("Format not supported:") + ToString(format));
		}
	}
}
