#include "StdAfx.h"

#include "filter.h"
#include "filter_int.h"
#include "lut_gamma.h"
#include "config.h"

namespace Filter {
	using namespace Geom;

	static const int GammaZero=64;

	void Adjust::Gamma(const FilterBuffer& source, FilterBuffer& dest, const Geom::RectInt& sourceRegion, const Geom::PointInt& destPos, const int gamma) {
		if (gamma == 10) return;
		COND_STRICT(source.BufferData != 0 && dest.BufferData != 0, Err::InvalidParam, TX("Null buffer not allowed."));
		COND_STRICT(gamma >= Img::MinGamma && gamma <= Img::MaxGamma, Err::InvalidParam, TX("Gamma out of bounds."));

		int y1 = Util::Max(0, sourceRegion.Top());
		int y2 = Util::Min(source.Dimensions.Height, sourceRegion.Bottom());
		int x1 = Util::Max(0, sourceRegion.Left());
		int x2 = Util::Min(source.Dimensions.Width, sourceRegion.Right());

		if ((x2 + destPos.X) > dest.Dimensions.Width)
			x2 = dest.Dimensions.Width - destPos.X;

		if ((y2 + destPos.Y) > dest.Dimensions.Height)
			y2 = dest.Dimensions.Height - destPos.Y;

		const Byte* gamma_table = c_gamma[gamma - Img::MinGamma];

		Byte* sourceCurrentScanlinePtr = source.BufferData;
		Byte* destCurrentScanlinePtr = dest.BufferData + destPos.X * 4 + destPos.Y * dest.Stride;

		for(int y = y1; y < y2; ++y) {
			DWord* sourceCurrentPixelPtr	= reinterpret_cast<DWord*>(sourceCurrentScanlinePtr);
			DWord* destCurrentPixelPtr		= reinterpret_cast<DWord*>(destCurrentScanlinePtr);

			for(int x = x1; x < x2; ++x) {
				DWord& scol = *(sourceCurrentPixelPtr++);

				Byte newA = static_cast<Byte>(Img::ChannelARGB8888Alpha(scol));

				Byte newR = gamma_table[Img::ChannelARGB8888Red(scol)];
				Byte newG = gamma_table[Img::ChannelARGB8888Green(scol)];
				Byte newB = gamma_table[Img::ChannelARGB8888Blue(scol)];

				*(destCurrentPixelPtr++) = Img::ToARGBDWORD(newA, newR, newG, newB);
			}
			sourceCurrentScanlinePtr += source.Stride;
			destCurrentScanlinePtr += dest.Stride;
		}
	}
}
