#include "StdAfx.h"

#include "filter.h"

namespace Filter {
	namespace Alpha {
		void Alpha::SolidColor(const FilterBuffer& source, FilterBuffer& dest, const Geom::RectInt& sourceRegion, const Geom::PointInt& destPos, const Img::Color& bgr) {
			COND_STRICT(source.BufferData != 0 && dest.BufferData != 0, Err::InvalidParam, TX("Null buffer not allowed."));

			int y1 = Util::Max(0, sourceRegion.Top());
			int y2 = Util::Min(source.Dimensions.Height, sourceRegion.Bottom());
			int x1 = Util::Max(0, sourceRegion.Left());
			int x2 = Util::Min(source.Dimensions.Width, sourceRegion.Right());

			if ((x2 + destPos.X) > dest.Dimensions.Width)
				x2 = dest.Dimensions.Width - destPos.X;

			if ((y2 + destPos.Y) > dest.Dimensions.Height)
				y2 = dest.Dimensions.Height - destPos.Y;

			DWord bgrDW	= Img::ToARGBDWORD(bgr);

			Byte* sourceCurrentScanlinePtr = source.BufferData;
			Byte* destCurrentScanlinePtr = dest.BufferData + destPos.X * 4 + destPos.Y * dest.Stride;

			for (int y = y1; y < y2; ++y) {
				DWord* sourceCurrentPixelPtr = reinterpret_cast<DWord*>(sourceCurrentScanlinePtr);
				DWord* destCurrentPixelPtr = reinterpret_cast<DWord*>(destCurrentScanlinePtr);

				for(int x = x1; x < x2; ++x) {
					DWord&	sp = *(sourceCurrentPixelPtr++);
					Byte	sa = Img::ChannelARGB8888Alpha(sp);

					*(destCurrentPixelPtr++) = Img::MixARGB8888ARGB8888(bgrDW, sp, sa);
				}
				sourceCurrentScanlinePtr += source.Stride;
				destCurrentScanlinePtr += dest.Stride;
			}
		}
	}
}
