#include "StdAfx.h"
#include "image.h"

namespace Img {
	using namespace Geom;
	using boost::recursive_mutex;
	using boost::condition;

	bool Image::IsPartial() {
		return false;
	}

	bool Image::IsFinished() {
		return isFinished;
	}

	bool Image::IsHeaderInformationValid() {
		return m_isHeaderDataValid;
	}

	void Image::SetHeaderData(const Geom::SizeInt& size, ImageComposer::Ptr composer) {
		boost::mutex::scoped_lock l(m_mxChangeState);
		COND_STRICT(composer.get(), Err::InvalidParam, TX("Composer was null."));

		m_composer = composer;
		m_size = size;
		m_isHeaderDataValid = true;
	}

	void Image::FinishImage(int loadTime) {
		this->loadTime = loadTime;
		isFinished = true;
	}

	void Image::Deallocate() {
		boost::mutex::scoped_lock l(m_mxChangeState);
		isFinished = false;
		m_composer.reset();
	}

	void Image::NextFrame() {
		//boost::recursive_mutex::scoped_lock lockState(m_mutexInspectState);
		boost::mutex::scoped_lock l(m_mxChangeState);

		if(IsHeaderInformationValid())
			m_composer->Advance();
	}

	int Image::Delay() {
		boost::mutex::scoped_lock l(m_mxChangeState);

		if(IsPartial())
			return -1;

		if(IsHeaderInformationValid() && m_composer.get())
			return m_composer->Delay();

		return -1;
	}

	void Image::RestartAnimation() {
		//boost::recursive_mutex::scoped_lock lockState(m_mutexInspectState);
		boost::mutex::scoped_lock l(m_mxChangeState);

		if(IsHeaderInformationValid() && m_composer)
			m_composer->Restart();
	}

	const Geom::SizeInt& Image::GetSize() const {
		//boost::recursive_mutex::scoped_lock lockState(m_mutexInspectState);
		return m_size;
	}

	Image::Image(String filename)
		:m_isHeaderDataValid(false),
		 isFinished(false),
		 loadTime(0)
	{}

	int Image::LoadTime() const {
		return loadTime;
	}

	Surface::Ptr Image::CurrentSurface() {
		boost::mutex::scoped_lock l(m_mxChangeState);

		if(IsHeaderInformationValid() && m_composer.get())
			return m_composer->RequestCurrentSurface();

		return Surface::Ptr();
	}
}
