#ifndef IMAGE_H
#define IMAGE_H

#include "types.h"
#include "imagecomposer.h"

namespace Img {
	class Image {
	public:
		typedef boost::shared_ptr<Image> Ptr;

	public:
		bool IsPartial(); // Occurs for cropped files, etc that can't be fully loaded.
		bool IsFinished(); // Finished = nothing more to do, is true also for corrupt images.
		bool IsHeaderInformationValid();

		void SetHeaderData(const Geom::SizeInt& size, ImageComposer::Ptr composer);
		void FinishImage(int loadTime);

		void Deallocate();
		Surface::Ptr CurrentSurface();

		// Valid when header has been successfully loaded at some point.
		const Geom::SizeInt& GetSize() const;

		// Valid when State >= StateHeader
		int Delay(); // Will return -1 for static or corrupt images

		void NextFrame();
		void RestartAnimation();

		int LoadTime() const;

		Image(String filename);

	private:
		ImageComposer::Ptr m_composer;

		boost::mutex m_mxChangeState;

		int loadTime;

		// Cached params
		bool isFinished;
		bool m_isHeaderDataValid;
		Geom::SizeInt m_size;
	};
}

#endif
