#include "StdAfx.h"

#include "surface.h"

namespace Img {
	Byte* Surface::LockedArea::Buffer() {
		return m_buffer;
	}

	void Surface::LockedArea::Unlock() {
		COND_WARNING(m_isLocked, Err::CriticalError, TX("Attempted to unlock an already unlocked area."));
		m_surface->UnlockSurface(m_method);
		m_isLocked = false;
	}

	Surface::LockedArea::LockedArea( Surface* surface, Byte* buffer, const Geom::RectInt& rect, LockMethod method, size_t stride )
		:m_surface(surface),
		m_buffer(buffer),
		m_rectLocked(rect),
		m_method(method),
		m_stride(stride),
		m_isLocked(true)
	{
		COND_WARNING(m_surface, Err::InvalidParam, TX("Surface was null."));
		COND_WARNING(m_buffer, Err::InvalidParam, TX("Locked buffer was null."));
		COND_WARNING(m_stride, Err::InvalidParam, TX("Stride not set."));
	}

	Surface::LockedArea::~LockedArea() {
		if (m_isLocked)
			Unlock();
	}
}
