#include "StdAfx.h"

#include "swsurface.h"
#include "config.h"
#include "surface_lock_swmr.h"

namespace Img {
	using Geom::SizeInt;
	using Geom::RectInt;
	using Geom::PointInt;

	bool SurfaceSoftware::onCreateSurface() {
		m_pDIData.reset();
		try {
			m_pDIData.reset(new Byte[static_cast<size_t>(Width()) * static_cast<size_t>(Height()) * static_cast<size_t>(PixelSize())]);
		}
		catch(std::bad_alloc&) {
			return false;
		}
		return true;
	}

	Byte* SurfaceSoftware::onLockSurface(const RectInt& region, Img::LockMethod ) {
		if(!m_pDIData) return 0;
		return &m_pDIData[region.Left() * PixelSize() + region.Top() * onStride()];
	}

	void SurfaceSoftware::onUnlockSurface() {}

	void SurfaceSoftware::onPaletteUpdate() {
		ForceDirty();
	}

	size_t SurfaceSoftware::onStride() const {
		return static_cast<size_t>(GetSize().Width) * static_cast<size_t>(PixelSize());
	}

	SurfaceSoftware::SurfaceSoftware() {
		SetLockStrategy(Img::LockStrategy::Ptr(new Img::LockStrategySingleWriterMultipleReaders));
	}

	Byte* SurfaceSoftware::DeviceIndependentData() {
		return m_pDIData.get();
	}

	void SurfaceSoftware::onDeallocate() {
		m_pDIData.reset();
	}
}
