#include "StdAfx.h"
#include "tiff_rawconverter.h"
#include "orz/bitconvert.h"

namespace Img {
	Byte* TiffRawConverter::PerformRequestDestinationBuffer(size_t bytes) {
		m_buffer.reset(new Byte[bytes]);
		return m_buffer.get();
	}

	Byte* TiffRawConverter::PerformReleaseDestinationBuffer(int widthInPixels, size_t bytesUsed, size_t* expandedSize) {
		COND_STRICT(m_buffer, Err::CriticalError, TX("Buffer not created."));

		*expandedSize = bytesUsed;

		Word samplesPerPixel = 0;
		TIFFGetField(m_tiff, TIFFTAG_SAMPLESPERPIXEL, &samplesPerPixel);
		Word bitsPerSample = 0;
		TIFFGetField(m_tiff, TIFFTAG_BITSPERSAMPLE, &bitsPerSample);

		if (samplesPerPixel == 1 && bitsPerSample < 8) {
			size_t stride = (widthInPixels * bitsPerSample + 7)/ 8;
			size_t numRows = bytesUsed / stride;

			*expandedSize = widthInPixels * numRows;
			ByteArray destData(new Byte[*expandedSize]);

			Byte* sourceRow = m_buffer.get();
			Byte* destRow = destData.get();

			for (size_t currentRow = 0; currentRow < numRows; ++currentRow){
				Util::ConvertToBytes(destRow, sourceRow, bitsPerSample, widthInPixels);
				destRow		+= widthInPixels;
				sourceRow	+= stride;
			}

			m_buffer = destData;
		}
		else if (samplesPerPixel == 3 && bitsPerSample == 8) {
			*expandedSize = (bytesUsed * 4) / 3;
			ByteArray destData(new Byte[*expandedSize]);

			Byte* sourceData = reinterpret_cast<Byte*>(m_buffer.get());
			DWord* destinationData = reinterpret_cast<DWord*>(destData.get());

			for (unsigned pixel = 0; pixel < (bytesUsed / 3); ++pixel) {
				*(destinationData++) = Img::ToARGBDWORD(Img::Color(0xff, sourceData[0], sourceData[1], sourceData[2]));
				sourceData += 3;
			}

			m_buffer = destData;
		}
		else if (samplesPerPixel == 4 && bitsPerSample == 8) {
			DWord* pDScan	= reinterpret_cast<DWord*>(m_buffer.get());

			for(size_t pixel = 0; pixel < (bytesUsed / 4); pixel++) {
				DWord currpix = pDScan[pixel];
				pDScan[pixel] = Img::ABGR_ARGB(currpix);
			}
		}
		else if (samplesPerPixel== 1 && bitsPerSample == 16) {
			Word* sourceData = reinterpret_cast<Word*>(m_buffer.get());

			*expandedSize = bytesUsed >> 1;
			ByteArray destData(new Byte[*expandedSize]);

			for(unsigned currSample = 0; currSample < *expandedSize; ++currSample)
				destData[currSample] = static_cast<Byte>(sourceData[currSample] >> 8);

			m_buffer = destData;
		}

		return m_buffer.get();
	}
}
