#include "StdAfx.h"
#include "bitconvert.h"

namespace Util {
	void BitsToBytes(Byte* dest, const Byte* source, DWord numBits) {
		int wholeBytes	= numBits >> 3;
		int extraBits	= numBits & 7;

		for(int curr = 0; curr < wholeBytes; ++curr) {
			Byte b = *(source++);

			for(int i = 0; i < 8; i++) {
				*(dest++) = (b & 0x80) >> 7;
				b <<= 1;
			}
		}

		// Transfer any remaining bits
		Byte b = *source;
		for(int i = 0; i < extraBits; i++) {
			*(dest++) = (b &0x80) >> 7;
			b <<= 1;
		}
	}

	void CrumbsToBytes(Byte* dest, const Byte* source, DWord numCrumbs) {
		int wholeBytes	= numCrumbs >> 2;
		int extraCrumbs	= numCrumbs & 3;

		for(int curr = 0; curr < wholeBytes; ++curr) {
			Byte b = *(source++);
			*(dest++) = (b &0xC0) >> 6;
			*(dest++) = (b &0x30) >> 4;
			*(dest++) = (b &0x0C) >> 2;
			*(dest++) = (b &0x03);
		}

		// Transfer any remaining crumbs
		Byte b = *source;

		for(int i = 0; i < extraCrumbs; i++) {
			*(dest++) = (b &0xC0) >> 6;
			b<<=2;
		}
	}

	void NibblesToBytes(Byte* dest, const Byte* source, DWord numNibbles) {
		int wholeBytes	= numNibbles >> 1;
		int extraNibble	= numNibbles & 1;

		for(int curr = 0; curr < wholeBytes; ++curr) {
			Byte b = *(source++);
			*(dest++) = (b & 0xf0) >> 4;
			*(dest++) = b & 0x0f;
		}
		// Transfer any remaining bits
		if (extraNibble != 0)
			*(dest++) = (*source & 0xf0) >> 4;
	}

	void ConvertToBytes(Byte* dest, const Byte* source, DWord sourceBitDepth, DWord numSourceWords) {
		switch(sourceBitDepth) {
		case 1:
			Util::BitsToBytes(dest, source, numSourceWords);
			break;
		case 2:
			Util::CrumbsToBytes(dest, source, numSourceWords);
			break;
		case 4:
			Util::NibblesToBytes(dest, source, numSourceWords);
			break;
		default:
			if (sourceBitDepth >= 1 && sourceBitDepth <= 8) {
				// Fail safe but somewhat slow and crops the words so can't be reliably used for >8 bits.
				StreamConverter sc;
				sc.ChangeWordSize(sourceBitDepth);
				COND_STRICT(sc.AddBytes(source, (numSourceWords * sourceBitDepth + 7) / 8), Err::CriticalError, TX("Couldn't insert source data into conversion stream."));
				COND_STRICT(sc.GetWordsAsByteMsbToLsb(dest, numSourceWords), Err::CriticalError, TX("Couldn't convert data."));
			}
			else
				DO_THROW(Err::Unsupported, TX("Unsupported word size: ") + ToString(sourceBitDepth));
		}
	}
}
