#ifndef CIRCULAR_BUFFER_H
#define CIRCULAR_BUFFER_H

namespace Util {
	template <typename T, int BufferSize = 1024>
	class CircularBuffer {
	public:
		void Clear() {
			m_readPosition = 0;
			m_numValid = 0;
		}

		T* PreparePassiveFeed() {
			m_readPosition = 0;

			if (m_numValid == 0)
				return m_data;

			memmove(m_data, m_data + m_readPosition, m_numValid);
			return m_data + m_numValid;
		}

		int SizePassiveFeed() {
			return BufferSize - m_numValid;
		}

		void PerformPassiveFeed(int numToFeed) {
			COND_STRICT(numToFeed <= (BufferSize - m_numValid), Err::CriticalError, TX("Diverted buffer overrun"));
			m_numValid += numToFeed;
		}

		bool RequestSingleItem(T* dest) {
			if (m_numValid == 0) return false;

			*dest = m_data[m_readPosition++];
			m_numValid--;

			return true;
		}

		CircularBuffer()
			:m_readPosition(0),
			 m_numValid(0)
		{}

	private:
		T						m_data[BufferSize];
		int						m_readPosition;
		int						m_numValid;
	};
}

#endif
