#ifndef FILE_H
#define FILE_H

#include "stream.h"

namespace IO {
	class FileReader {
	public:
		const String Name() const;

		bool Open();
		void Close();
		FileInt Size();

		Stream* GetStream();

		size_t Read(void* buf, size_t size, size_t items);
		void ReadFull(void* buf, size_t size);

		void Seek(FileInt position, SeekMethod m);

		FileInt Position() const;

		bool IsOpen() const;

		typedef shared_ptr<FileReader> Ptr;

		FileReader(const String& name);
		FileReader(Stream::Ptr s);

	private:
		FileReader(const FileReader& b);
		Stream::Ptr m_stream;
	};

	class FileWriter {
	public:
		bool Open(const String& name, bool append=false);
		void Close();

		size_t Write(const void* buf, size_t size, size_t items);

		FileWriter();
		~FileWriter();

	private:
		std::FILE*					m_file;
	};
}

#endif
