#include "StdAfx.h"
#include "file.h"
#include "fileops.h"

namespace IO {
	using boost::mutex;

	mutex g_mutexSHFileOperation;

	struct HANDLETOMAPPINGS {
		UINT              uNumberOfMappings;  // Number of mappings in the array.
		LPSHNAMEMAPPING   lpSHNameMapping;    // Pointer to the array of mappings.
	};

	bool DoFileExist(const String& file) {
		if (file==TX("")) return false;
		IO::FileReader r(file);
		return r.Open();

		/*FILE* f = 0;
		if ((_wfopen_s(&f, file.c_str(), TX("rb")) == 0) && (f != 0)) {
			fclose(f);
			return true;
		}
		return false;*/
	}

	bool DoPathExist(const String& file) {
		DWORD dwAttr=GetFileAttributes(file.c_str());

		if (dwAttr==INVALID_FILE_ATTRIBUTES)		
			return false;

		if (dwAttr&FILE_ATTRIBUTE_DIRECTORY)
			return true;
		else
			return false;
	}

	static shared_array<Char> ToTerminatedCharArray(const String& in) {
		size_t len = in.length() + 2; // Two extra due to _two_ terminating chars.
		shared_array<Char> pFromString(new Char[len]);

		// Copy source-file into the struct and apply the extra terminator.
		errno_t copyRet = wcscpy_s(pFromString.get(), len, in.c_str());
		COND_WARNING(copyRet == 0, Err::CriticalError, TX("Couldn't copy string to buffer."));

		pFromString.get()[len - 1] = 0;	// Shuts up vs2005 code analysis

		return pFromString;
	}

	bool performDeleteRecycle(const String& file, bool doRecycle, HWND hwnd) {
		//COND_STRICT(hwnd != 0, Err::InvalidParam, TX("hwnd was null."));
		SHFILEOPSTRUCT sfop;
		ZeroMemory(&sfop, sizeof(sfop));

		shared_array<Char> pFromString = ToTerminatedCharArray(file);
		sfop.pFrom = pFromString.get();
		sfop.wFunc = FO_DELETE;
		sfop.fFlags = FOF_SILENT;

		if(hwnd == 0)
			sfop.fFlags |= FOF_NOCONFIRMATION;

		if (doRecycle)
			sfop.fFlags |= FOF_ALLOWUNDO;

		sfop.hwnd = hwnd;

		mutex::scoped_lock l(g_mutexSHFileOperation);
		return (SHFileOperation(&sfop) == 0) && (sfop.fAnyOperationsAborted == false);
	}

	bool FileDelete(const String& file, HWND hwnd) {
		return performDeleteRecycle(file, false, hwnd);
	}

	bool FileRecycle(const String& file, HWND hwnd) {
		return performDeleteRecycle(file, true, hwnd);
	}

	bool Move(const String& old_path, const String& new_path) {
		// Not updated
		assert(false);
		return (::MoveFile(old_path.c_str(), new_path.c_str()) != 0);
	}

	String Rename(const String& old_name, const String& new_name, HWND hwnd) {
		SHFILEOPSTRUCT sfop;
		ZeroMemory(&sfop, sizeof(sfop));

		shared_array<Char> pOldName = ToTerminatedCharArray(old_name);
		shared_array<Char> pNewName = ToTerminatedCharArray(new_name);
		sfop.pFrom = pOldName.get();
		sfop.pTo = pNewName.get();
		sfop.wFunc = FO_RENAME;
		sfop.fFlags = FOF_SILENT | FOF_ALLOWUNDO | FOF_WANTMAPPINGHANDLE;
		sfop.hwnd = hwnd;

		if(hwnd == 0)
			sfop.fFlags |= FOF_NOCONFIRMATION;

//		int ticket = pause_folders(GetPath(old_name));

		mutex::scoped_lock l(g_mutexSHFileOperation);
		bool toRet = (SHFileOperation(&sfop) == 0) && (sfop.fAnyOperationsAborted == false);
		l.unlock();

		String resulting_name = new_name;

		HANDLETOMAPPINGS* s = reinterpret_cast<HANDLETOMAPPINGS*>(sfop.hNameMappings);
		if(s && s->uNumberOfMappings == 1)
			resulting_name = s->lpSHNameMapping[0].pszNewPath;

		SHFreeNameMappings(sfop.hNameMappings);

		/*if (toRet)
			renamed_file(old_name, resulting_name);

		restore_folders(ticket);*/
		if(!toRet) return TX("");
		return resulting_name;
	}

	bool SupportRecycle() {
		return true;
	}
}
