#include "StdAfx.h"
#include "intl.h"
#include "sysinfo.h"

namespace Intl {
	boost::signals2::signal<void()> OnLanguageChanged;

	Language m_currLang = LangEnglish;

	// Make sure no identifiers are valid by default
	int m_minID  = 1;
	int m_maxID  = 0;
	const TableEntry* m_pTable = 0;

	void LanguageTable(const TableEntry* pTable, int minID, int maxID) {
		m_pTable	= pTable;
		m_minID		= minID;
		m_maxID		= maxID;
	}

	void CurrentLanguage(const Language& lang) {
		if (m_currLang != lang) {
			m_currLang = lang;
			OnLanguageChanged();
		}
	}

	const Language CurrentLanguage() {
		return m_currLang;
	}

	Language SystemLanguage() {
		return Sys::Info::SystemLanguage();
	}

	const Char*		GetString(int id) {
		return GetStringLang(id, m_currLang);
	}

	const Char*		GetStringLang(int id, Language lang) {
		COND_STRICT((id >= m_minID) && (id <= m_maxID), Err::CriticalError, TX("Invalid string identifier requested."));
		COND_STRICT(lang < FinalLang, Err::CriticalError, TX("Invalid language identifier requested."));

		return m_pTable[id].langs[lang];
	}
}
