#ifndef OBSERVER_H
#define OBSERVER_H

namespace Util {
	template <typename T> class Subject;

	template <class T>
	class Observer {
	public:
		Observer() {}
		virtual ~Observer()=0 {}
		virtual void Signal(T* value)=0;

		typedef T OurSubject;
	};

	template <typename T>
	class Subject {
	private:
		typedef std::list<Observer<T>* > ObserverList;

	public:
		void Attach(Observer<T>* pObserver) {
			m_observers.push_back(pObserver);
			++m_count;
		}

		void Detach(Observer<T>* pObserver) {
			assert(m_count > 0);

			// Find and remove the observer
			ObserverList::iterator i = std::find(m_observers.begin(), m_observers.end(), pObserver);
			assert(i != m_observers.end());
			m_observers.erase(i);
			--m_count;
		}

		void Notify() {
			for(ObserverList::iterator i = m_observers.begin(); i != m_observers.end(); ++i)
				(*i)->Signal(dynamic_cast<T *>(this));
		}

		Subject():m_count(0) {}
		virtual ~Subject()=0 {}

	private:
		ObserverList m_observers;
		int m_count;
	};
}

#endif
