#include "StdAfx.h"
#include "stream.h"

namespace IO {
	String Stream::Name() const {
		return performName();
	}

	bool Stream::Open() {
		COND_STRICT(!IsOpen(), Err::FileAlreadyOpen, TX("Stream is already open."));
		return performOpen();
	}

	bool Stream::IsOpen() const {
		return performIsOpen();
	}

	void Stream::Close() {
		performClose();
	}

	size_t Stream::Read(void* buf, size_t size, size_t items) {
		COND_STRICT(IsOpen(), Err::FileNotOpen, TX("Stream is not open."));
		return performRead(buf, size, items);
	}

	void Stream::Seek(FileInt position, SeekMethod m) {
		COND_STRICT(IsOpen(), Err::FileNotOpen, TX("Stream is not open."));
		return performSeek(position, m);
	}

	FileInt Stream::Position() const {
		COND_STRICT(IsOpen(), Err::FileNotOpen, TX("Stream is not open."));
		return performPosition();
	}

	FileInt Stream::Size() {
		COND_STRICT(IsOpen(), Err::FileNotOpen, TX("Stream is not open."));
		return performSize();
	}

	Stream::~Stream() {}
}
