#ifndef STREAM_H
#define STREAM_H

namespace Err {
	struct IOException:public Exception {
		IOException(const String& msg):Exception(TX("IO: ") + msg) {};
	};

	struct FileNotOpen:public IOException {
		FileNotOpen(const String& msg):IOException(TX("FileNotOpen: ")+msg) {}
	};

	struct FileAlreadyOpen:public IOException {
		FileAlreadyOpen(const String& msg):IOException(TX("FileAlreadyOpen: ")+msg) {}
	};

	struct EndOfFile:public IOException {
		EndOfFile(const String& msg):IOException(TX("End of file: ")+msg) {}
	};
}


namespace IO {
	enum SeekMethod {
		SeekMethodBegin,
		SeekMethodCurrent,
		SeekMethodEnd
	};

	enum OpenErrorCode {
		OpenAccessViolation,
		OpenFileNotFound,
		OpenSucceeded,
	};

	class Stream:boost::noncopyable {
	public:
		String Name() const;
		bool Open();
		bool IsOpen() const;
		void Close();
		size_t Read(void* buf, size_t size, size_t items);
		void Seek(FileInt position, SeekMethod m);
		FileInt Position() const;
		FileInt Size();

		virtual ~Stream();

		typedef boost::shared_ptr<Stream> Ptr;

	private:
		virtual String performName() const=0;
		virtual bool performOpen()=0;
		virtual bool performIsOpen() const=0;
		virtual void performClose()=0;
		virtual size_t performRead(void* buf, size_t size, size_t items)=0;
		virtual void performSeek(FileInt position, SeekMethod m)=0;
		virtual FileInt performPosition() const=0;
		virtual FileInt performSize()=0;
	};
}

#endif
