#include "StdAfx.h"
#include "stream_mem.h"

namespace IO {
	using boost::recursive_mutex;

	size_t StreamMemory::performRead(void* buf, size_t size, size_t items) {
		recursive_mutex::scoped_lock l(m_mutexAccess);
		COND_STRICT(m_pos <= m_size, Err::CriticalError, TX("Position was out of bounds, invalid object state."));
		int numItems = Util::Min<size_t>(items, (m_size - m_pos) / size);
		memcpy(buf, m_data + m_pos, size * numItems);
		m_pos += size * numItems;
		return numItems;
	}

	FileInt StreamMemory::performPosition() const {
		return m_pos;
	}

	void StreamMemory::performClose() {
		recursive_mutex::scoped_lock l(m_mutexAccess);
		m_data = 0;
		m_pos = 0;
		m_size = 0;
		m_isOpen = false;
	}

	bool StreamMemory::performOpen() {
		recursive_mutex::scoped_lock l(m_mutexAccess);
		m_isOpen = (m_data != 0);
		m_pos = 0;
		return m_isOpen;
	}

	void StreamMemory::performSeek(FileInt position, SeekMethod m) {
		recursive_mutex::scoped_lock l(m_mutexAccess);

		if(m == SeekMethodBegin)
			m_pos = position;
		else if (m == SeekMethodCurrent)
			m_pos += position;
		else if (m == SeekMethodEnd)
			m_pos = m_size;

		if(m_pos > m_size) m_pos = m_size;
	}

	bool StreamMemory::performIsOpen() const {
		return m_isOpen;
	}

	FileInt StreamMemory::performSize() {
		return m_size;
	}

	String StreamMemory::performName() const {
		return TX("Memory buffer");
	}

	StreamMemory::StreamMemory(Byte* data, size_t size):m_data(data), m_size(size), m_pos(0), m_isOpen(false) {}
}
