#include "StdAfx.h"
#include "stream_windows.h"

namespace IO {
	StreamWindows::StreamWindows(IStream* i):m_pStream(i) {
	}

	StreamWindows::StreamWindows(const String& filename) {
		if(FAILED(SHCreateStreamOnFile(filename.c_str(), STGM_READ | STGM_SHARE_DENY_WRITE, &m_pStream)))
			DO_THROW(Err::InvalidParam, TX("Could not create stream for file: ") + filename);
	}

	StreamWindows::~StreamWindows() {
		if(m_pStream)
			m_pStream->Release();
		m_pStream = 0;
	}

	bool StreamWindows::performOpen() {
		return true;
	}

	bool StreamWindows::performIsOpen() const {
		return true;
	}

	void StreamWindows::performClose() {}

	size_t StreamWindows::performRead(void* buf, size_t size, size_t items) {
		ULONG bytesRead;
		ULONG bytesToRead = size * items;
		m_pStream->Read(buf, bytesToRead, &bytesRead);
		if(bytesRead != bytesToRead) {
			LARGE_INTEGER delta;
			delta.QuadPart = bytesRead - bytesToRead;
			m_pStream->Seek(delta, STREAM_SEEK_CUR, 0);
			return bytesRead / size;
		}
		return items;
		//m_pStream->Revert()
	}

	void StreamWindows::performSeek(FileInt position, SeekMethod m) {
		DWORD dwOrigin = STREAM_SEEK_SET;

		if(m == IO::SeekMethodBegin)
			dwOrigin = STREAM_SEEK_SET;
		else if (m == IO::SeekMethodCurrent)
			dwOrigin = STREAM_SEEK_CUR;
		else if (m == IO::SeekMethodEnd)
			dwOrigin = STREAM_SEEK_END;
		LARGE_INTEGER pos;
		pos.QuadPart = position;
		m_pStream->Seek(pos, m, 0);
	}

	FileInt StreamWindows::performPosition() const {
		ULARGE_INTEGER pos;
		LARGE_INTEGER dummy;
		dummy.QuadPart = 0;
		m_pStream->Seek(dummy, STREAM_SEEK_CUR, &pos);
		return pos.QuadPart;
	}

	FileInt StreamWindows::performSize() {
		STATSTG ssg;
		m_pStream->Stat(&ssg, STATFLAG_NONAME);
		return ssg.cbSize.QuadPart;
	}

	String StreamWindows::performName() const {
		STATSTG ssg;
		m_pStream->Stat(&ssg, STATFLAG_DEFAULT);
		String toReturn(ssg.pwcsName);
		CoTaskMemFree(ssg.pwcsName);
		return toReturn;
	}
}
