#include "StdAfx.h"
#include "streamconv.h"

namespace Util {
	void StreamConverter::AddByte(const Byte b) {
		m_source.push_back(b);
	}

	bool StreamConverter::AddBytes(const Byte* pBuf, size_t bufSize) {
		if (!pBuf) return false;
		for(size_t i = 0; i < bufSize; i++)
			m_source.push_back(pBuf[i]);

		return true;
	}

	int StreamConverter::GetWord() {
		unsigned int currChar = 0;
		unsigned int dstPos = 0;
		unsigned int copied = 0;

		do {
			COND_STRICT(!m_source.empty(), Err::CriticalError, TX("Out of data!"));

			Byte toCopy = Util::Min((Byte)(8 - m_srcPos), static_cast<Byte>(m_wordSize - copied));
			Word mask = static_cast<Word>((1 << toCopy) - 1) << m_srcPos;
			Byte currPart = static_cast<Byte>((m_source.front() & mask) >> m_srcPos);

			currChar |=  (currPart << dstPos);

			m_srcPos += toCopy;
			if (m_srcPos > 7) {
				m_srcPos = 0;
				m_source.pop_front();
			}

			dstPos += toCopy;
			copied += toCopy;
		}
		while (copied < m_wordSize);

		return static_cast<short>(currChar);
	}

	int StreamConverter::GetWordMsbToLsb() {
		unsigned int currChar = 0;
		unsigned int dstPos = 0;
		unsigned int copied = 0;

		do {
			COND_STRICT(m_source.empty() == false, Err::CriticalError, TX("Out of data!"));

			Byte toCopy = Util::Min((Byte)(8 - m_srcPos), static_cast<Byte>(m_wordSize - copied));
			Word mask = static_cast<Word>(((1 << toCopy) - 1) << (8 - (toCopy + m_srcPos)));
			Byte currPart = static_cast<Byte>((m_source.front() & mask) >> (8 - (toCopy + m_srcPos)));

			currChar |=  (currPart << (m_wordSize - (toCopy + dstPos)));

			m_srcPos += toCopy;
			if (m_srcPos > 7) {
				m_srcPos = 0;
				m_source.pop_front();
			}

			dstPos += toCopy;
			copied += toCopy;
		}
		while (copied < m_wordSize);

		return static_cast<short>(currChar);
	}

	bool StreamConverter::GetWordsAsByte(Byte* pDest, size_t num) {
		if (!pDest) return false;

		for(size_t i = 0; i < num; i++)
			pDest[i] = static_cast<Byte>(GetWord());

		return true;
	}

	bool StreamConverter::GetWordsAsByteMsbToLsb(Byte* pDest, size_t num) {
		if (!pDest) return false;

		for(size_t i = 0; i < num; i++)
			pDest[i] = static_cast<Byte>(GetWordMsbToLsb());

		return true;
	}

	unsigned int StreamConverter::CurrentWordSize() const {
		return m_wordSize;
	}

	void StreamConverter::ChangeWordSize(unsigned int newBitCount) {
		m_wordSize = newBitCount;
	}

	bool StreamConverter::IsWordsLeft() const {
		return (static_cast<unsigned int>(m_source.size() * 8 - m_srcPos) >= m_wordSize);
	}

	StreamConverter::StreamConverter(Byte destBitCount) :m_srcPos(0), m_wordSize(destBitCount) {}
}
