#include "StdAfx.h"
#include "threadpool.h"
#include "thread.h"

namespace Util {
	void Threadpool::AddTask(Task func) {
		boost::mutex::scoped_lock l(m_mxTasks);
		m_tasks.push_back(func);
		m_cdWorkAvail.notify_all();
	}

	void Threadpool::JoinAll() {
		boost::mutex::scoped_lock l(m_mxTasks);
		while(m_numRunning || !m_tasks.empty())
			m_cdIdle.wait(l);
	}

	Threadpool::Threadpool(size_t numThreads):m_terminate(false), m_numRunning(0) {
		boost::mutex::scoped_lock l(m_mxTasks);
		m_threads.resize(numThreads);
		foreach(ThreadPtr& p, m_threads)
			p.reset(new boost::thread(bind(&Threadpool::Worker, this)));
	}

	Threadpool::~Threadpool() {
		JoinAll();

		boost::mutex::scoped_lock l(m_mxTasks);
		m_terminate = true;
		m_cdWorkAvail.notify_all();
		l.unlock();
		foreach(ThreadPtr t, m_threads)
			t->join();
	}

	void Threadpool::Worker() {
		boost::mutex::scoped_lock l(m_mxTasks);
		while(!m_terminate) {
			m_cdIdle.notify_all();
			while(m_tasks.empty() && !m_terminate)
				m_cdWorkAvail.wait(l);
			if(m_terminate) return;

			m_numRunning++;
			Task t=m_tasks.front();
			m_tasks.pop_front();
			l.unlock();
			t();
			l.lock();
			m_numRunning--;
		}
	}
}
