#ifndef ORZ_TYPES_H
#define ORZ_TYPES_H

#define foreach BOOST_FOREACH

// Conditional expression is constant
#pragma warning(disable: 4127)

typedef unsigned char Byte;
typedef unsigned short Word;
typedef unsigned int DWord;
typedef __int64 FileInt;

typedef int Int;

typedef __int16 Int16;
typedef __int32 Int32;

typedef wchar_t WChar;
typedef std::wstring WString;


typedef char AChar;
typedef std::string AString;

#ifdef UNICODE
typedef WString String;
typedef WChar Char;
#define TX_(t) L##t
#else
typedef AString					String;
typedef AChar					Char;
#define TX_(t) t
#endif

#define TX(t) TX_(t)

typedef std::basic_stringstream<Char, std::char_traits<Char>, std::allocator<Char> > StringStream;

String ToString(Byte b);
String ToString(int i);
String ToString(DWord d);
String ToString(const char* i);

template <typename T>
String ToString(T i) {
	StringStream ss;
	ss << i;
	return ss.str();
}

AString ToAString(String s);

template <typename T>
T FromString(const String& s) {
	StringStream ss(s);
	T i;
	ss >> i;
	return i;
}

String UTF8ToString(const char* utf8);

int RoundCast(float rhs);

bool IsDecimal(const String& s);

// Modified design by contract macros.
// I don't want to remove ALL pre/post checks in release build but it can be
// very nifty to remove the silliest ones

String DoThrowBuildDescription(Char* filename, int line, Char* functionName, String description);

#define DO_THROW(exception, description) do { throw exception(DoThrowBuildDescription(TX(__FILE__), __LINE__, TX(__FUNCTION__), description)); } while (0)
#define COND_STRICT(condition, exception, description) do { if ((condition) == false) DO_THROW(exception, (description)); } while (0)

#ifdef _DEBUG
#define COND_WARNING(condition, exception, description) COND_STRICT((condition), exception, (description))
#else
#define COND_WARNING(condition, exception, description) do { sizeof(condition); } while(0)
#endif

namespace Util {
	template <typename T> const T& Min(const T& a, const T& b) { return a<b?a:b; }
	template <typename T> const T& Min(const T& a, const T& b, const T& c) {
		if(a < b) {
			if(c < a)
				return c;
			return a;
		}
		else if(c < b)
			return c;
		return b;
	}
	template <typename T> const T& Max(const T& a, const T& b) { return a>b?a:b; }
	template <typename T> const T& Max(const T& a, const T& b, const T& c) { return Max(a, Max(b, c)); }


	template <typename T> const T& Constrain(const T& lower, const T& val, const T& upper) {
		return Util::Min<T>(Util::Max<T>(lower, val), upper);
	}
}

namespace Num {
	const float FEps = std::numeric_limits<float>::epsilon();
	const double DEps = std::numeric_limits<double>::epsilon();
	const long double LDEpsilon = std::numeric_limits<long double>::epsilon();

	const float Pi = 3.14159265358979323846f;
	const float Pi_2 = 1.57079632679489661923f;
	const float Pi_4 = 0.785398163397448309616f;
};

String ToUpper(const String& s);
String ToLower(const String& s);

#endif
