#include "StdAfx.h"

#include "illa/filter.h"

#include "share_filterbuffer.h"
#include "illa/filter_int.h"

using namespace Filter;
using Geom::RectInt;
using Geom::PointInt;
using Geom::SizeInt;
using namespace Tst;

SUITE(Filters_Resample)
{
	TEST(RegionCropping)
	{
		RectInt result = CropResampleRegions(
			RectInt(PointInt(0, 362), SizeInt(500, 253)),
			SizeInt(500, 250),
			SizeInt(500, 253),
			1.0f);

		CHECK(result.Height() == 0);
	}

	TEST_FIXTURE(ReferenceRotationFixture, Empty_Targets)
	{
		// Filter to empty targets. All of these should fail.
		m_bufferDestination.filterBuffer.BufferData = 0;

		for(int i = 0; i < FilterNum; i++)
			CHECK_THROW(Scale::AutoFilter(FilterMode(i), m_bufferSource, m_bufferDestination.filterBuffer, m_regionDefault, Img::FormatARGB8888, 1.0), Err::InvalidParam);
	}

	TEST_FIXTURE(ReferenceRotationFixture, Empty_Sources)
	{
		// Filter from empty sources. All of these should fail.
		m_bufferSource.BufferData = 0;

		for(int i = 0; i < FilterNum; i++)
			CHECK_THROW(Scale::AutoFilter(FilterMode(i), m_bufferSource, m_bufferDestination.filterBuffer, m_regionDefault, Img::FormatARGB8888, 1.0), Err::InvalidParam);
	}

	TEST_FIXTURE(ReferenceRotationFixture, Invalid_Filters)
	{
		// Use invalid filters
		CHECK_THROW(Scale::AutoFilter(FilterUndefined, m_bufferSource, m_bufferDestination.filterBuffer, m_regionDefault, Img::FormatARGB8888, 1.3f), Err::InvalidParam);
		CHECK_THROW(Scale::AutoFilter(FilterNum, m_bufferSource, m_bufferDestination.filterBuffer, m_regionDefault, Img::FormatARGB8888, 1.3f), Err::InvalidParam);

		// Use invalid formats
		for(int i = 0; i < FilterNum; i++)
		{
			CHECK_THROW(Scale::AutoFilter(FilterMode(i), m_bufferSource, m_bufferDestination.filterBuffer, m_regionDefault, Img::FormatNum, (i==FilterDirectCopy?1.0f:1.3f)), Err::InvalidParam);
			CHECK_THROW(Scale::AutoFilter(FilterMode(i), m_bufferSource, m_bufferDestination.filterBuffer, m_regionDefault, Img::Format(Img::FormatNum+3), (i==FilterDirectCopy?1.0f:1.3f)), Err::InvalidParam);
		}
	}

	TEST_FIXTURE(PaddedBuffersFixture, DirectCopy_NonMatchingStride)
	{
		Scale::AutoFilter(FilterDirectCopy, m_bufferSource.filterBuffer, m_bufferDestination.filterBuffer, m_regionDefault, Img::FormatARGB8888, 1.0f);
		CHECK_EQUAL(CheckPaddingUntouched(m_bufferDestination, DestinationPadding), true);
	}

	TEST_FIXTURE(PaddedBuffersFixture, Nearest_NonMatchingStride)
	{
		Scale::AutoFilter(FilterNearestNeighbor, m_bufferSource.filterBuffer, m_bufferDestination.filterBuffer, m_regionDefault, Img::FormatARGB8888, 2.0f);
		CHECK_EQUAL(CheckPaddingUntouched(m_bufferDestination, DestinationPadding), true);
	}

	TEST_FIXTURE(PaddedBuffersFixture, Bilinear_NonMatchingStride)
	{
		Scale::AutoFilter(FilterBilinear, m_bufferSource.filterBuffer, m_bufferDestination.filterBuffer, m_regionDefault, Img::FormatARGB8888, 2.0f);
		CHECK_EQUAL(CheckPaddingUntouched(m_bufferDestination, DestinationPadding), true);
	}
}
