#include "StdAfx.h"
#include "orz/bitconvert.h"

SUITE(BitConvert)
{
	/*

		Nibble

	*/

	TEST(NibbleMatch)
	{
		Byte dest[2];
		Byte source = 0x98;
		Util::NibblesToBytes(dest, &source, 2);
		CHECK(dest[0] == 0x09);
		CHECK(dest[1] == 0x08);
	}

	TEST(NibbleNotMatch)
	{
		Byte dest[3];
		Byte source[2] = { 0x98, 0x23 };
		Util::NibblesToBytes(dest, source, 3);
		CHECK(dest[0] == 0x09);
		CHECK(dest[1] == 0x08);
		CHECK(dest[2] == 0x02);
	}

	TEST(NibbleSingle)
	{
		Byte dest;
		Byte source = 0xCD;
		Util::NibblesToBytes(&dest, &source, 1);
		CHECK(dest == 0x0C);
	}

	TEST(NibbleNullNum)
	{
		Byte dest = 0x33;
		Byte source = 0xff;
		Util::NibblesToBytes(&dest, &source, 0);
		CHECK(dest == 0x33);
	}

	/*

	Crumb

	*/


	TEST(CrumbMatch)
	{
		Byte dest[4];
		Byte source = 0x98;
		Util::CrumbsToBytes(dest, &source, 4);
		CHECK(dest[0] == 0x02);
		CHECK(dest[1] == 0x01);
		CHECK(dest[2] == 0x02);
		CHECK(dest[3] == 0x00);
	}

	TEST(CrumbNotMatch)
	{
		Byte dest[4] = { 0, 0, 0, 0xff};
		Byte source = 0x98;
		Util::CrumbsToBytes(dest, &source, 3);
		CHECK(dest[0] == 0x02);
		CHECK(dest[1] == 0x01);
		CHECK(dest[2] == 0x02);
		CHECK(dest[3] == 0xff);
	}

	TEST(CrumbSingle)
	{
		Byte dest;
		Byte source = 0xCD;
		Util::CrumbsToBytes(&dest, &source, 1);
		CHECK(dest == 0x03);
	}

	TEST(CrumbNullNum)
	{
		Byte dest = 0x33;
		Byte source = 0xff;
		Util::CrumbsToBytes(&dest, &source, 0);
		CHECK(dest == 0x33);
	}


	/*

	Bit

	*/


	TEST(BitMatch)
	{
		Byte dest[8];
		Byte source = 0x98;
		Util::BitsToBytes(dest, &source, 8);
		CHECK(dest[0] == 0x01);
		CHECK(dest[1] == 0x00);
		CHECK(dest[2] == 0x00);
		CHECK(dest[3] == 0x01);
		CHECK(dest[4] == 0x01);
		CHECK(dest[5] == 0x00);
		CHECK(dest[6] == 0x00);
		CHECK(dest[7] == 0x00);
	}

	TEST(BitNotMatch)
	{
		Byte dest[5] = { 0, 0, 0, 0xff};
		Byte source = 0x98;
		Util::BitsToBytes(dest, &source, 3);
		CHECK(dest[0] == 0x01);
		CHECK(dest[1] == 0x00);
		CHECK(dest[2] == 0x00);
		CHECK(dest[3] == 0xff);
	}

	TEST(BitSingle)
	{
		Byte dest = 0;
		Byte source = 0xCD;
		Util::BitsToBytes(&dest, &source, 1);
		CHECK(dest == 0x01);
	}

	TEST(BitNullNum)
	{
		Byte dest = 0x33;
		Byte source = 0xff;
		Util::BitsToBytes(&dest, &source, 0);
		CHECK(dest == 0x33);
	}
}
